@@ -19,15 +19,14 @@
 #define CONFIGOBJECT_H
 
 #include <qstring.h>
-#include <q3ptrlist.h>
-#include <q3valuelist.h>
 #include <qfile.h>
-#include <q3textstream.h>
 #include <qkeysequence.h>
 #include <qdom.h>
 #include <qmap.h>
 #include <QHash>
 
+#include "util/debug.h"
+
 /*
 typedef enum {
     MIDI_EMPTY            = 0,
@@ -72,7 +71,7 @@ public:
     ConfigValue();
     ConfigValue(QString _value);
     ConfigValue(int _value);
-    inline ConfigValue(QDomNode /* node */) { qFatal("ConfigValue from QDomNode not implemented here"); }
+    inline ConfigValue(QDomNode /* node */) { reportFatalErrorAndQuit("ConfigValue from QDomNode not implemented here"); }
     void valCopy(const ConfigValue _value);
 
     QString value;
@@ -86,7 +85,7 @@ public:
     ConfigValueKbd();
     ConfigValueKbd(QString _value);
     ConfigValueKbd(QKeySequence key);
-    inline ConfigValueKbd(QDomNode /* node */) { qFatal("ConfigValueKbd from QDomNode not implemented here"); }
+    inline ConfigValueKbd(QDomNode /* node */) { reportFatalErrorAndQuit("ConfigValueKbd from QDomNode not implemented here"); }
     void valCopy(const ConfigValueKbd v);
     friend bool operator==(const ConfigValueKbd & s1, const ConfigValueKbd & s2);
 
@@ -96,35 +95,41 @@ public:
 template <class ValueType> class ConfigOption
 {
   public:
-    ConfigOption() {};
+    ConfigOption() { val = NULL; key = NULL;};
     ConfigOption(ConfigKey *_key, ValueType *_val) { key = _key ; val = _val; };
+    virtual ~ConfigOption() {
+        delete key;
+        delete val;
+    }
     ValueType *val;
     ConfigKey *key;
 };
 
-template <class ValueType> class ConfigObject
-{
+template <class ValueType> class ConfigObject {
   public:
     ConfigKey key;
     ValueType value;
     ConfigOption<ValueType> option;
 
     ConfigObject(QString file);
-	ConfigObject(QDomNode node);
+    ConfigObject(QDomNode node);
     ~ConfigObject();
     ConfigOption<ValueType> *set(ConfigKey, ValueType);
     ConfigOption<ValueType> *get(ConfigKey key);
+    bool exists(ConfigKey key);
     ConfigKey *get(ValueType v);
     QString getValueString(ConfigKey k);
+    QString getValueString(ConfigKey k, const QString& default_string);
 
     void clear();
     void reopen(QString file);
     void Save();
-    QString getConfigPath();
+    QString getResourcePath();
+    QString getSettingsPath() const;
 
   protected:
-    Q3PtrList< ConfigOption<ValueType> > list;
-    QString filename;
+    QList< ConfigOption<ValueType>* > m_list;
+    QString m_filename;
 
     /** Loads and parses the configuration file. Returns false if the file could
       * not be opened; otherwise true. */
