@@ -20,13 +20,11 @@
 
 #include <QObject>
 #include <QEvent>
-#include <q3ptrlist.h>
-//#include <qaccel.h>
-#include <q3ptrqueue.h>
 #include <QMutex>
-#include "midi/midimessage.h"
+
 #include "configobject.h"
 #include "controlobjectthread.h"
+#include "controllers/midi/midimessage.h"
 
 class QWidget;
 class ConfigKey;
@@ -41,7 +39,7 @@ struct QueueObjectThread
 struct QueueObjectMidi
 {
     ControlObject *pControlObject;
-    MidiCategory category;
+    MidiOpCode opcode;
     double value;
 };
 
@@ -63,14 +61,23 @@ class ControlObject : public QObject
     Q_OBJECT
 public:
     ControlObject();
-    ControlObject(ConfigKey key, bool bIgnoreNops=true);
+    ControlObject(ConfigKey key, bool bIgnoreNops=true, bool track=false);
+    ControlObject(const QString& group, const QString& item, bool bIgnoreNops=true);
     virtual ~ControlObject();
     /** Connect two control objects dest and src, so each time src is updated, so is dest. */
     static bool connectControls(ConfigKey src, ConfigKey dest);
     /** Disonnect a control object. */
     static bool disconnectControl(ConfigKey key);
     /** Returns a pointer to the ControlObject matching the given ConfigKey */
-    static ControlObject *getControl(ConfigKey key);
+    static ControlObject* getControl(const ConfigKey& key);
+    static inline ControlObject* getControl(const QString& group, const QString& item) {
+        ConfigKey key(group, item);
+        return getControl(key);
+    }
+
+    // Adds all ControlObjects that currently exist to pControlList
+    static void getControls(QList<ControlObject*>* pControlsList);
+
     /** Used to add a pointer to the corresponding ControlObjectThread of this ControlObject */
     void addProxy(ControlObjectThread *pControlObjectThread);
     // To get rid of a proxy when the corresponding object is being deleted for example
@@ -79,9 +86,9 @@ public:
       * happend, otherwise false. */
     bool updateProxies(ControlObjectThread *pProxyNoUpdate=0);
     /** Return the key of the object */
-    ConfigKey getKey();
+    inline ConfigKey getKey() { return m_key; }
     /** Return the value of the ControlObject */
-    double get();
+    inline double get() { return m_dValue; }
     /** Add to value. Not thread safe. */
     void add(double dValue);
     /** Subtract from value. Not thread safe. */
@@ -91,13 +98,19 @@ public:
     /** Queue a control change from a widget. Thread safe. Blocking. */
     void queueFromThread(double dValue, ControlObjectThread *pControlObjectThread=0);
     /** Queue a control change from MIDI. Thread safe. Blocking. */
-    void queueFromMidi(MidiCategory c, double v);
+    void queueFromMidi(MidiOpCode o, double v);
     /** Return a ControlObject value, corresponding to the widget input value. Thread safe. */
     virtual double getValueFromWidget(double dValue);
     /** Return a widget value corresponding to the ControlObject input value. Thread safe. */
     virtual double getValueToWidget(double dValue);
     /** get value (range 0..127) **/
     virtual double GetMidiValue();
+    virtual void setDefaultValue(double dValue) {
+        m_dDefaultValue = dValue;
+    }
+    virtual double defaultValue() const {
+        return m_dDefaultValue;
+    }
 
 public slots:
     /** Sets the value of the object and updates associated proxy objects. Not thread safe. */
@@ -111,21 +124,27 @@ protected:
     /** Sets the value of the object. Not thread safe. */
     virtual void setValueFromEngine(double dValue);
     /** Called when a widget has changed value. Not thread safe. */
-    virtual void setValueFromMidi(MidiCategory, double v);
+    virtual void setValueFromMidi(MidiOpCode o, double v);
     /** Called when another thread has changed value. Not thread safe. */
     virtual void setValueFromThread(double dValue);
 
 protected:
     /** The actual value of the controller */
     double m_dValue;
+    double m_dDefaultValue;
     /** Key of the object */
-    ConfigKey m_Key;
+    ConfigKey m_key;
 
 private:
     // Whether to ignore set/add/sub()'s which would have no effect
     bool m_bIgnoreNops;
+    // Whether to track value changes with the stats framework.
+    bool m_bTrack;
+    QString m_trackKey;
+    int m_trackType;
+    int m_trackFlags;
     /** List of associated proxy objects */
-    Q3PtrList<ControlObjectThread> m_qProxyList;
+    QList<ControlObjectThread*> m_qProxyList;
     /** Mutex for the proxy list */
     QMutex m_qProxyListMutex;
 
@@ -136,12 +155,12 @@ private:
     /** Mutex protecting access to the queues */
     static QMutex m_sqQueueMutexMidi, m_sqQueueMutexThread, m_sqQueueMutexChanges;
     /** Queue holding control changes from MIDI */
-    static Q3PtrQueue<QueueObjectMidi> m_sqQueueMidi;
+    static QQueue<QueueObjectMidi*> m_sqQueueMidi;
     /** Queues holding control changes from other application threads and from widgets */
-    static Q3PtrQueue<QueueObjectThread> m_sqQueueThread;
+    static QQueue<QueueObjectThread*> m_sqQueueThread;
     /** Queue holding ControlObjects that has changed, but not been syncronized with it's
      * associated ControlObjectProxy objects. */
-    static Q3PtrQueue<ControlObject> m_sqQueueChanges;
+    static QQueue<ControlObject*> m_sqQueueChanges;
 };
 
 
