@@ -15,42 +15,35 @@
 *                                                                         *
 ***************************************************************************/
 
-#include <qapplication.h>
+#include <QApplication>
+#include <QtDebug>
+
 #include "controlobjectthread.h"
 #include "controlobject.h"
 
-
-QWaitCondition ControlObjectThread::m_sqWait;
-QMutex ControlObjectThread::m_sqMutex;
-Q3PtrQueue<ControlObjectThread> ControlObjectThread::m_sqQueue;
-
-ControlObjectThread::ControlObjectThread(ControlObject * pControlObject)
-: m_dValue          (0.0)
-, m_pControlObject  (pControlObject)
-{
-    // Update associated ControlObject
-    if (m_pControlObject == NULL) return;
-    Q_ASSERT(m_pControlObject);
-    m_pControlObject->addProxy(this);
-
-    connect(m_pControlObject, SIGNAL(destroyed()), this, SLOT(slotParentDead()));
-
-    // Initialize value
-    m_dValue = m_pControlObject->get();
+ControlObjectThread::ControlObjectThread(ControlObject* pControlObject, QObject* pParent)
+        : QObject(pParent),
+          m_dValue(0.0),
+          m_pControlObject(pControlObject) {
+    // Register with the associated ControlObject
+    if (m_pControlObject != NULL) {
+        m_pControlObject->addProxy(this);
+        connect(m_pControlObject, SIGNAL(destroyed()),
+                this, SLOT(slotParentDead()));
+        // Initialize value
+        m_dValue = m_pControlObject->get();
+    }
     emitValueChanged();
 }
 
-ControlObjectThread::~ControlObjectThread()
-{
-    if (m_pControlObject) 
-    {
+ControlObjectThread::~ControlObjectThread() {
+    if (m_pControlObject) {
         // Our parent is still around, make sure it doesn't send us any more events
         m_pControlObject->removeProxy(this);
     }
 }
 
-double ControlObjectThread::get()
-{
+double ControlObjectThread::get() {
     m_dataMutex.lock();
     double v = m_dValue;
     m_dataMutex.unlock();
@@ -58,99 +51,61 @@ double ControlObjectThread::get()
     return v;
 }
 
-void ControlObjectThread::slotSet(double v)
-{
+void ControlObjectThread::slotSet(double v) {
     m_dataMutex.lock();
     m_dValue = v;
     m_dataMutex.unlock();
 
-    updateControlObject();
+    updateControlObject(v);
 }
 
-bool ControlObjectThread::setExtern(double v)
-{
+bool ControlObjectThread::setExtern(double v) {
     bool result = false;
 
     if (m_dataMutex.tryLock()) {
         m_dValue = v;
         result = true;
         m_dataMutex.unlock();
+        emitValueChanged();
     }
 
-    // if ( m_sqMutex.tryLock() )
-    // {
-    //     if( m_dataMutex.tryLock() )
-    //     {
-    //         m_sqQueue.enqueue(this);
-    //         m_dValue = v;
-            
-    //         m_dataMutex.unlock();
-    //         m_sqMutex.unlock();
-            
-    //         result = true;
-    //     }
-    //     else
-    //     {
-    //         m_sqMutex.unlock();
-    //     }
-    // }
-
     return result;
 }
 
-bool ControlObjectThread::update()
-{
-    bool result = false;
-
-    m_sqMutex.lock();
-    ControlObjectThread* p = m_sqQueue.dequeue();
-    m_sqMutex.unlock();
-    
-    if (p)
-    {
-        p->emitValueChanged();
-        result = true;
-    }
-    
-    return result;
-}
-
-void ControlObjectThread::emitValueChanged()
-{
+void ControlObjectThread::emitValueChanged() {
     emit(valueChanged(get()));
 }
 
-void ControlObjectThread::add(double v)
-{
+void ControlObjectThread::add(double v) {
     m_dataMutex.lock();
-    m_dValue += v;
+    double newValue = m_dValue + v;
+    m_dValue = newValue;
     m_dataMutex.unlock();
 
-    updateControlObject();
+    updateControlObject(newValue);
 }
 
-void ControlObjectThread::sub(double v)
-{
+void ControlObjectThread::sub(double v) {
     m_dataMutex.lock();
-    m_dValue -= v;
+    double newValue = m_dValue - v;
+    m_dValue = newValue;
     m_dataMutex.unlock();
 
-    updateControlObject();
+    updateControlObject(newValue);
 }
 
-void ControlObjectThread::updateControlObject()
-{
-    m_pControlObject->queueFromThread(get(), this);
+void ControlObjectThread::updateControlObject(double v) {
+    if (m_pControlObject) {
+        m_pControlObject->queueFromThread(v, this);
+    }
 }
 
-void ControlObjectThread::slotParentDead() 
-{
+void ControlObjectThread::slotParentDead() {
     // Now we've got a chance of avoiding segfaults with judicious
     // use of if(m_pControlObject)
-    m_pControlObject = 0;
+    m_pControlObject = NULL;
 }
 
-ControlObject* ControlObjectThread::getControlObject() 
-{
+ControlObject* ControlObjectThread::getControlObject() {
    return m_pControlObject;
 }
