@@ -17,7 +17,6 @@
 #ifndef DEFS_H
 #define DEFS_H
 
-// Version number moved to defs_version.h to avoid lots of rebuilding
 #define MIXXX_PROMO_DIR "promo"
 
 #include <math.h>
@@ -50,6 +49,17 @@ const int STEPSIZE = 1024; //WINDOWSIZE/2; //WINDOWSIZE/STEPSIZE must result in
 /** Maximum buffer length to each EngineObject::process call */
 const int MAX_BUFFER_LEN = 160000;
 
+#ifndef PATH_MAX
+#ifndef MAX_PATH
+// http://msdn.microsoft.com/en-us/library/windows/desktop/aa365247%28v=vs.85%29.aspx
+const int MAX_PATH = 260;
+#endif
+// Use POSIX name for MAX_PATH
+enum {
+	PATH_MAX = MAX_PATH
+};
+#endif
+
 // Various fixed constants
 // static CSAMPLE pi     = acos(-1.0); // Conflicts with macx headers
 // static CSAMPLE two_pi = (2.f*acos(-1.f));
@@ -77,5 +87,14 @@ inline double zap_denormal(double x)
 #define math_min(a,b)            (((a) < (b)) ? (a) : (b))
 #endif
 
+// MSVC 2005/2008 needs these
+#ifndef fmax
+#define fmax math_max
+#endif
+
+#ifndef fmin
+#define fmin math_min
+#endif
+
 #endif
 
