@@ -19,6 +19,10 @@
 
 #define MIXXX_WEBSITE_URL       "http://www.mixxx.org"
 #define MIXXX_SUPPORT_URL       "http://www.mixxx.org/support.php"
+#define MIXXX_FEEDBACK_URL      "https://docs.google.com/forms/d/1Zbx-VgdcSIuTTmy9UalNZhtDypSJdSLvRVZ0MKOuLu4/viewform"
+#define MIXXX_TRANSLATION_URL   "https://translations.launchpad.net/mixxx"
+#define MIXXX_MANUAL_URL        "http://mixxx.org/manual/1.11"
+#define MIXXX_MANUAL_FILENAME   "Mixxx-Manual.pdf"
 #define MIXXX_PROMO_URL         "http://promo.mixxx.org/"
 
 #endif
