@@ -20,14 +20,14 @@
 
 #include "ui_dlgprefvinyldlg.h"
 #include "configobject.h"
-#include "soundmanager.h"
-#include "vinylcontrolsignalwidget.h"
+#include "vinylcontrol/vinylcontrolsignalwidget.h"
+#include "controlobjectthreadmain.h"
 
 class QWidget;
 class PlayerProxy;
 class ControlObject;
 class ControlObjectThreadMain;
-
+class VinylControlManager;
 
 /**
   *@author Stefan Langhammer
@@ -37,54 +37,31 @@ class ControlObjectThreadMain;
 class DlgPrefVinyl : public QWidget, Ui::DlgPrefVinylDlg  {
     Q_OBJECT
 public:
-    DlgPrefVinyl(QWidget *parent, SoundManager* soundman, ConfigObject<ConfigValue> *_config);
+    DlgPrefVinyl(QWidget *pParent, VinylControlManager *m_pVCMan, ConfigObject<ConfigValue> *_config);
     ~DlgPrefVinyl();
 
 public slots:
     /** Update widget */
     void slotUpdate();
     void slotApply();
-	void ChannelsSlotApply();
-    void slotComboBoxDeviceDeck1Change();
-    void slotComboBoxDeviceDeck2Change();
-    void refreshDeck1Channels();
-    void refreshDeck2Channels();
-	void EnableRelativeModeSlotApply();
-	void EnableScratchModeSlotApply();
-	void VinylTypeSlotApply();
-	void AutoCalibrationSlotApply();
+    void VinylTypeSlotApply();
     void VinylGainSlotApply();
-    void enableValidComboBoxes();
-    void updateSignalQuality1(double value);
-    void updateSignalQuality2(double value);
-    void updateInputLevelLeft1(double value);
-    void updateInputLevelRight1(double value);
-    void updateInputLevelLeft2(double value);
-    void updateInputLevelRight2(double value);
     void slotClose();
     void slotShow();
 
 signals:
-    void apply();
 private:
-	void applySoundDeviceChanges();
-
     VinylControlSignalWidget m_signalWidget1;
     VinylControlSignalWidget m_signalWidget2;
 
 
     /** Pointer to player device */
     //PlayerProxy *player;
-    SoundManager* m_pSoundManager;
+    VinylControlManager* m_pVCManager;
     /** Pointer to config object */
     ConfigObject<ConfigValue> *config;
-    /** Indicates the strength of the timecode signal on each input */
-    ControlObjectThreadMain* m_timecodeQuality1;
-    ControlObjectThreadMain* m_timecodeQuality2;
-    ControlObjectThreadMain* m_vinylControlInput1L;
-    ControlObjectThreadMain* m_vinylControlInput1R;
-    ControlObjectThreadMain* m_vinylControlInput2L;
-    ControlObjectThreadMain* m_vinylControlInput2R;
+    ControlObjectThreadMain m_COSpeed1;
+    ControlObjectThreadMain m_COSpeed2;
 };
 
 #endif
