@@ -12,13 +12,12 @@
 #include "cachingreader.h"
 #include "mathstuff.h"
 
-#define NUM_HOT_CUES 32
-
 CueControl::CueControl(const char * _group,
                        ConfigObject<ConfigValue> * _config) :
         EngineControl(_group, _config),
         m_bPreviewing(false),
         m_bPreviewingHotcue(false),
+        m_bHotcueCancel(false),
         m_pPlayButton(ControlObject::getControl(ConfigKey(_group, "play"))),
         m_iCurrentlyPreviewingHotcues(0),
         m_iNumHotCues(NUM_HOT_CUES),
@@ -28,160 +27,165 @@ CueControl::CueControl(const char * _group,
 
     m_pTrackSamples = ControlObject::getControl(ConfigKey(_group, "track_samples"));
 
+    m_pQuantizeEnabled = ControlObject::getControl(ConfigKey(_group, "quantize"));
+
+    m_pNextBeat = ControlObject::getControl(ConfigKey(_group, "beat_next"));
+    m_pClosestBeat = ControlObject::getControl(ConfigKey(_group, "beat_closest"));
+
     m_pCuePoint = new ControlObject(ConfigKey(_group, "cue_point"));
-    m_pCueMode = new ControlObject(ConfigKey(_group,"cue_mode"));
+    m_pCueMode = new ControlObject(ConfigKey(_group, "cue_mode"));
     m_pCuePoint->set(-1);
 
     m_pCueSet = new ControlPushButton(ConfigKey(_group, "cue_set"));
     connect(m_pCueSet, SIGNAL(valueChanged(double)),
-            this, SLOT(cueSet(double)));
+            this, SLOT(cueSet(double)),
+            Qt::DirectConnection);
 
     m_pCueGoto = new ControlPushButton(ConfigKey(_group, "cue_goto"));
     connect(m_pCueGoto, SIGNAL(valueChanged(double)),
-            this, SLOT(cueGoto(double)));
+            this, SLOT(cueGoto(double)),
+            Qt::DirectConnection);
+
+    m_pCueGotoAndPlay =
+            new ControlPushButton(ConfigKey(_group, "cue_gotoandplay"));
+    connect(m_pCueGotoAndPlay, SIGNAL(valueChanged(double)),
+            this, SLOT(cueGotoAndPlay(double)),
+            Qt::DirectConnection);
 
     m_pCueGotoAndStop =
             new ControlPushButton(ConfigKey(_group, "cue_gotoandstop"));
     connect(m_pCueGotoAndStop, SIGNAL(valueChanged(double)),
-            this, SLOT(cueGotoAndStop(double)));
+            this, SLOT(cueGotoAndStop(double)),
+            Qt::DirectConnection);
 
     m_pCueSimple = new ControlPushButton(ConfigKey(_group, "cue_simple"));
     connect(m_pCueSimple, SIGNAL(valueChanged(double)),
-            this, SLOT(cueSimple(double)));
+            this, SLOT(cueSimple(double)),
+            Qt::DirectConnection);
 
     m_pCuePreview = new ControlPushButton(ConfigKey(_group, "cue_preview"));
     connect(m_pCuePreview, SIGNAL(valueChanged(double)),
-            this, SLOT(cuePreview(double)));
+            this, SLOT(cuePreview(double)),
+            Qt::DirectConnection);
 
     m_pCueCDJ = new ControlPushButton(ConfigKey(_group, "cue_cdj"));
     connect(m_pCueCDJ, SIGNAL(valueChanged(double)),
-            this, SLOT(cueCDJ(double)));
+            this, SLOT(cueCDJ(double)),
+            Qt::DirectConnection);
 
     m_pCueDefault = new ControlPushButton(ConfigKey(_group, "cue_default"));
     connect(m_pCueDefault, SIGNAL(valueChanged(double)),
-            this, SLOT(cueDefault(double)));
+            this, SLOT(cueDefault(double)),
+            Qt::DirectConnection);
+
+    connect(m_pPlayButton, SIGNAL(valueChanged(double)),
+            this, SLOT(playFromCuePreview(double)),
+            Qt::DirectConnection);
 }
 
 CueControl::~CueControl() {
-}
-
-ConfigKey CueControl::keyForControl(int hotcue, QString name) {
-    ConfigKey key;
-    key.group = getGroup();
-    // Add one to hotcue so that we dont have a hotcue_0
-    key.item = QString("hotcue_%1_%2").arg(hotcue+1).arg(name);
-    return key;
+    delete m_pCuePoint;
+    delete m_pCueMode;
+    delete m_pCueSet;
+    delete m_pCueGoto;
+    delete m_pCueGotoAndPlay;
+    delete m_pCueGotoAndStop;
+    delete m_pCueSimple;
+    delete m_pCuePreview;
+    delete m_pCueCDJ;
+    delete m_pCueDefault;
+    while (m_hotcueControl.size() > 0) {
+        HotcueControl* pControl = m_hotcueControl.takeLast();
+        delete pControl;
+    }
 }
 
 void CueControl::createControls() {
     for (int i = 0; i < m_iNumHotCues; ++i) {
-        ControlObject* hotcuePosition = new ControlObject(
-            keyForControl(i, "position"));
-        connect(hotcuePosition, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcuePositionChanged(double)));
-        hotcuePosition->set(-1);
-        m_controlMap[hotcuePosition] = i;
-        m_hotcuePosition.append(hotcuePosition);
+        HotcueControl* pControl = new HotcueControl(getGroup(), i);
 
-        ControlObject* hotcueEnabled = new ControlObject(
-            keyForControl(i, "enabled"));
-        hotcueEnabled->set(0);
-        m_controlMap[hotcueEnabled] = i;
-        m_hotcueEnabled.append(hotcueEnabled);
+        connect(pControl, SIGNAL(hotcuePositionChanged(HotcueControl*, double)),
+                this, SLOT(hotcuePositionChanged(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueSet(HotcueControl*, double)),
+                this, SLOT(hotcueSet(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueGoto(HotcueControl*, double)),
+                this, SLOT(hotcueGoto(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueGotoAndPlay(HotcueControl*, double)),
+                this, SLOT(hotcueGotoAndPlay(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueGotoAndStop(HotcueControl*, double)),
+                this, SLOT(hotcueGotoAndStop(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueActivate(HotcueControl*, double)),
+                this, SLOT(hotcueActivate(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueActivatePreview(HotcueControl*, double)),
+                this, SLOT(hotcueActivatePreview(HotcueControl*, double)),
+                Qt::DirectConnection);
+        connect(pControl, SIGNAL(hotcueClear(HotcueControl*, double)),
+                this, SLOT(hotcueClear(HotcueControl*, double)),
+                Qt::DirectConnection);
 
-        ControlObject* hotcueSet = new ControlPushButton(
-            keyForControl(i, "set"));
-        m_controlMap[hotcueSet] = i;
-        connect(hotcueSet, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueSet(double)));
-        m_hotcueSet.append(hotcueSet);
-
-        ControlObject* hotcueGoto = new ControlPushButton(
-            keyForControl(i, "goto"));
-        m_controlMap[hotcueGoto] = i;
-        connect(hotcueGoto, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueGoto(double)));
-        m_hotcueGoto.append(hotcueGoto);
-
-        ControlObject* hotcueGotoAndStop = new ControlPushButton(
-            keyForControl(i, "gotoandstop"));
-        m_controlMap[hotcueGotoAndStop] = i;
-        connect(hotcueGotoAndStop, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueGotoAndStop(double)));
-        m_hotcueGotoAndStop.append(hotcueGotoAndStop);
-
-        ControlObject* hotcueActivate = new ControlPushButton(
-            keyForControl(i, "activate"));
-        m_controlMap[hotcueActivate] = i;
-        connect(hotcueActivate, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueActivate(double)));
-        m_hotcueActivate.append(hotcueActivate);
-
-        ControlObject* hotcueActivatePreview = new ControlPushButton(
-            keyForControl(i, "activate_preview"));
-        m_controlMap[hotcueActivatePreview] = i;
-        connect(hotcueActivatePreview, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueActivatePreview(double)));
-        m_hotcueActivatePreview.append(hotcueActivatePreview);
-
-        ControlObject* hotcueClear = new ControlPushButton(
-            keyForControl(i, "clear"));
-        m_controlMap[hotcueClear] = i;
-        connect(hotcueClear, SIGNAL(valueChanged(double)),
-                this, SLOT(hotcueClear(double)));
-        m_hotcueClear.append(hotcueClear);
-
-        m_hotcue.append(NULL);
+        m_hotcueControl.append(pControl);
     }
 }
 
 void CueControl::attachCue(Cue* pCue, int hotCue) {
-    Q_ASSERT(hotCue >= 0 && hotCue < m_iNumHotCues);
-    if (m_hotcue[hotCue] != NULL) {
-        detachCue(hotCue);
+    if (hotCue < 0 || hotCue >= m_iNumHotCues) {
+        return;
     }
-    Q_ASSERT(m_hotcue[hotCue] == NULL);
-    m_hotcue[hotCue] = pCue;
+    HotcueControl* pControl = m_hotcueControl[hotCue];
+    if (pControl->getCue() != NULL) {
+        detachCue(pControl->getHotcueNumber());
+    }
+    pControl->setCue(pCue);
     connect(pCue, SIGNAL(updated()),
-            this, SLOT(cueUpdated()));
+            this, SLOT(cueUpdated()),
+            Qt::DirectConnection);
 
-    m_hotcuePosition[hotCue]->set(pCue->getPosition());
-    m_hotcueEnabled[hotCue]->set(pCue->getPosition() == -1 ? 0.0 : 1.0);
+    pControl->getPosition()->set(pCue->getPosition());
+    pControl->getEnabled()->set(pCue->getPosition() == -1 ? 0.0 : 1.0);
 }
 
 void CueControl::detachCue(int hotCue) {
-    Q_ASSERT(hotCue >= 0 && hotCue < m_iNumHotCues);
-    Cue* pCue = m_hotcue[hotCue];
+    if (hotCue < 0 || hotCue >= m_iNumHotCues) {
+        return;
+    }
+    HotcueControl* pControl = m_hotcueControl[hotCue];
+    Cue* pCue = pControl->getCue();
     if (!pCue)
         return;
     disconnect(pCue, 0, this, 0);
-    m_hotcue[hotCue] = NULL;
-    m_hotcuePosition[hotCue]->set(-1);
-    m_hotcueEnabled[hotCue]->set(0);
+    pControl->setCue(NULL);
+    pControl->getPosition()->set(-1);
+    pControl->getEnabled()->set(0);
 }
 
-void CueControl::loadTrack(TrackPointer pTrack) {
-    Q_ASSERT(pTrack);
-
+void CueControl::trackLoaded(TrackPointer pTrack) {
     QMutexLocker lock(&m_mutex);
     if (m_pLoadedTrack)
-        unloadTrack(m_pLoadedTrack);
+        trackUnloaded(m_pLoadedTrack);
+
+    if (!pTrack) {
+        return;
+    }
 
     m_pLoadedTrack = pTrack;
     connect(pTrack.data(), SIGNAL(cuesUpdated()),
-            this, SLOT(trackCuesUpdated()));
+            this, SLOT(trackCuesUpdated()),
+            Qt::DirectConnection);
 
     Cue* loadCue = NULL;
-    Cue* otherCue = NULL;
     const QList<Cue*>& cuePoints = pTrack->getCuePoints();
     QListIterator<Cue*> it(cuePoints);
     while (it.hasNext()) {
         Cue* pCue = it.next();
         if (pCue->getType() == Cue::LOAD) {
             loadCue = pCue;
-        } else if (pCue->getType() == Cue::CUE) {
-            otherCue = pCue;
-        } else {
+        } else if (pCue->getType() != Cue::CUE) {
             continue;
         }
         int hotcue = pCue->getHotCue();
@@ -189,11 +193,6 @@ void CueControl::loadTrack(TrackPointer pTrack) {
             attachCue(pCue, hotcue);
     }
 
-    // Prefer to load a LOAD cue, otherwise load any cue.
-    if (loadCue == NULL && otherCue) {
-        loadCue = otherCue;
-    }
-
     if (loadCue != NULL) {
         m_pCuePoint->set(loadCue->getPosition());
     } else {
@@ -204,17 +203,16 @@ void CueControl::loadTrack(TrackPointer pTrack) {
         ConfigKey("[Controls]","CueRecall")).toInt();
     //If cue recall is ON in the prefs, then we're supposed to seek to the cue
     //point on song load. Note that cueRecall == 0 corresponds to "ON", not OFF.
+    double loadCuePoint = 0;
     if (loadCue && cueRecall == 0) {
-        double loadCuePoint = loadCue->getPosition();
-
-        // Need to unlock before emitting any signals to prevent deadlock.
-        lock.unlock();
-
-        emit(seekAbs(loadCuePoint));
+        loadCuePoint = loadCue->getPosition();
     }
+    // Need to unlock before emitting any signals to prevent deadlock.
+    lock.unlock();
+    emit(seekAbs(loadCuePoint));
 }
 
-void CueControl::unloadTrack(TrackPointer pTrack) {
+void CueControl::trackUnloaded(TrackPointer pTrack) {
     QMutexLocker lock(&m_mutex);
     disconnect(pTrack.data(), 0, this, 0);
     for (int i = 0; i < m_iNumHotCues; ++i) {
@@ -247,7 +245,7 @@ void CueControl::unloadTrack(TrackPointer pTrack) {
 }
 
 void CueControl::cueUpdated() {
-    QMutexLocker lock(&m_mutex);
+    //QMutexLocker lock(&m_mutex);
     // We should get a trackCuesUpdated call anyway, so do nothing.
 }
 
@@ -268,7 +266,8 @@ void CueControl::trackCuesUpdated() {
 
         int hotcue = pCue->getHotCue();
         if (hotcue != -1) {
-            Cue* pOldCue = m_hotcue[hotcue];
+            HotcueControl* pControl = m_hotcueControl[hotcue];
+            Cue* pOldCue = pControl->getCue();
 
             // If the old hotcue is different than this one.
             if (pOldCue != pCue) {
@@ -278,15 +277,15 @@ void CueControl::trackCuesUpdated() {
                 attachCue(pCue, hotcue);
             } else {
                 // If the old hotcue is the same, then we only need to update
-                double dOldPosition = m_hotcuePosition[hotcue]->get();
-                double dOldEnabled = m_hotcueEnabled[hotcue]->get();
+                double dOldPosition = pControl->getPosition()->get();
+                double dOldEnabled = pControl->getEnabled()->get();
                 double dPosition = pCue->getPosition();
                 double dEnabled = dPosition == -1 ? 0.0 : 1.0;
                 if (dEnabled != dOldEnabled) {
-                    m_hotcueEnabled[hotcue]->set(dEnabled);
+                    pControl->getEnabled()->set(dEnabled);
                 }
                 if (dPosition != dOldPosition) {
-                    m_hotcuePosition[hotcue]->set(dPosition);
+                    pControl->getPosition()->set(dPosition);
                 }
             }
             // Add the hotcue to the list of active hotcues
@@ -301,13 +300,7 @@ void CueControl::trackCuesUpdated() {
     }
 }
 
-int CueControl::senderHotcue(QObject* pSender) {
-    Q_ASSERT(pSender);
-    Q_ASSERT(m_controlMap.contains(pSender));
-    return m_controlMap[pSender];
-}
-
-void CueControl::hotcueSet(double v) {
+void CueControl::hotcueSet(HotcueControl* pControl, double v) {
     //qDebug() << "CueControl::hotcueSet" << v;
 
     if (!v)
@@ -317,18 +310,32 @@ void CueControl::hotcueSet(double v) {
     if (!m_pLoadedTrack)
         return;
 
-    int hotcue = senderHotcue(sender());
+    int hotcue = pControl->getHotcueNumber();
     detachCue(hotcue);
     Cue* pCue = m_pLoadedTrack->addCue();
-    pCue->setPosition(getCurrentSample());
+    double cuePosition =
+            (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
+            floorf(m_pClosestBeat->get()) : floorf(getCurrentSample());
+    if (!even(cuePosition))
+        cuePosition--;
+    pCue->setPosition(cuePosition);
     pCue->setHotCue(hotcue);
     pCue->setLabel("");
     pCue->setType(Cue::CUE);
     // TODO(XXX) deal with spurious signals
     attachCue(pCue, hotcue);
+
+    // If quantize is enabled and we are not playing, jump to the cue point
+    // since it's not necessarily where we currently are. TODO(XXX) is this
+    // potentially invalid for vinyl control?
+    bool playing = m_pPlayButton->get() > 0;
+    if (!playing && m_pQuantizeEnabled->get() > 0.0) {
+        lock.unlock();  // prevent deadlock.
+        emit(seekAbs(cuePosition));
+    }
 }
 
-void CueControl::hotcueGoto(double v) {
+void CueControl::hotcueGoto(HotcueControl* pControl, double v) {
     if (!v)
         return;
 
@@ -336,8 +343,7 @@ void CueControl::hotcueGoto(double v) {
     if (!m_pLoadedTrack)
         return;
 
-    int hotcue = senderHotcue(sender());
-    Cue* pCue = m_hotcue[hotcue];
+    Cue* pCue = pControl->getCue();
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
@@ -350,7 +356,7 @@ void CueControl::hotcueGoto(double v) {
     }
 }
 
-void CueControl::hotcueGotoAndStop(double v) {
+void CueControl::hotcueGotoAndStop(HotcueControl* pControl, double v) {
     if (!v)
         return;
 
@@ -358,8 +364,7 @@ void CueControl::hotcueGotoAndStop(double v) {
     if (!m_pLoadedTrack)
         return;
 
-    int hotcue = senderHotcue(sender());
-    Cue* pCue = m_hotcue[hotcue];
+    Cue* pCue = pControl->getCue();
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
@@ -368,84 +373,12 @@ void CueControl::hotcueGotoAndStop(double v) {
         int position = pCue->getPosition();
         if (position != -1) {
             m_pPlayButton->set(0.0);
-
             emit(seekAbs(position));
         }
     }
 }
 
-void CueControl::hotcueActivate(double v) {
-    //qDebug() << "CueControl::hotcueActivate" << v;
-
-    QMutexLocker lock(&m_mutex);
-
-    if (!m_pLoadedTrack)
-        return;
-
-    int hotcue = senderHotcue(sender());
-    Cue* pCue = m_hotcue[hotcue];
-
-    lock.unlock();
-
-    if (pCue) {
-        if (v) {
-            if (pCue->getPosition() == -1) {
-                hotcueSet(v);
-            } else {
-                if (!m_bPreviewingHotcue && m_pPlayButton->get() == 1.0f) {
-                    hotcueGoto(v);
-                } else {
-                    hotcueActivatePreview(v);
-                }
-            }
-        } else {
-            if (pCue->getPosition() != -1) {
-                hotcueActivatePreview(v);
-            }
-        }
-    } else {
-        if (v) {
-            hotcueSet(v);
-        }
-    }
-}
-
-void CueControl::hotcueActivatePreview(double v) {
-    QMutexLocker lock(&m_mutex);
-    if (!m_pLoadedTrack)
-        return;
-    int hotcue = senderHotcue(sender());
-    Cue* pCue = m_hotcue[hotcue];
-
-    if (v) {
-        if (pCue && pCue->getPosition() != -1) {
-            m_iCurrentlyPreviewingHotcues++;
-            int iPosition = pCue->getPosition();
-            m_pPlayButton->set(1.0);
-            m_bPreviewingHotcue = true;
-
-            // Need to unlock before emitting any signals to prevent deadlock.
-            lock.unlock();
-
-            emit(seekAbs(iPosition));
-        }
-    } else {
-        if (m_bPreviewingHotcue && pCue && pCue->getPosition() != -1) {
-            if (--m_iCurrentlyPreviewingHotcues == 0) {
-                int iPosition = pCue->getPosition();
-                m_pPlayButton->set(0.0);
-                m_bPreviewingHotcue = false;
-
-                // Need to unlock before emitting any signals to prevent deadlock.
-                lock.unlock();
-
-                emit(seekAbs(iPosition));
-            }
-        }
-    }
-}
-
-void CueControl::hotcueClear(double v) {
+void CueControl::hotcueGotoAndPlay(HotcueControl* pControl, double v) {
     if (!v)
         return;
 
@@ -453,30 +386,150 @@ void CueControl::hotcueClear(double v) {
     if (!m_pLoadedTrack)
         return;
 
-    int hotcue = senderHotcue(sender());
+    Cue* pCue = pControl->getCue();
+
+    // Need to unlock before emitting any signals to prevent deadlock.
+    lock.unlock();
 
-    Cue* pCue = m_hotcue[hotcue];
     if (pCue) {
-        pCue->setHotCue(-1);
+        int position = pCue->getPosition();
+        if (position != -1) {
+            emit(seekAbs(position));
+            m_pPlayButton->set(1.0);
+        }
     }
-
-    detachCue(hotcue);
 }
 
-void CueControl::hotcuePositionChanged(double newPosition) {
+void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
+    //qDebug() << "CueControl::hotcueActivate" << v;
+
+    QMutexLocker lock(&m_mutex);
+
+    if (!m_pLoadedTrack)
+        return;
+
+    Cue* pCue = pControl->getCue();
+
+    lock.unlock();
+
+    if (pCue) {
+        if (v) {
+            m_bHotcueCancel = false;
+            if (pCue->getPosition() == -1) {
+                hotcueSet(pControl, v);
+            } else {
+                if (!m_bPreviewingHotcue && m_pPlayButton->get() == 1.0f) {
+                    hotcueGoto(pControl, v);
+                } else {
+                    hotcueActivatePreview(pControl, v);
+                }
+            }
+        } else {
+            if (pCue->getPosition() != -1) {
+                hotcueActivatePreview(pControl, v);
+            }
+        }
+    } else {
+        if (v) {
+            // just in case
+            m_bHotcueCancel = false;
+            hotcueSet(pControl, v);
+        } else if (m_bPreviewingHotcue) {
+            // The cue is non-existent, yet we got a release for it and are
+            // currently previewing a hotcue. This is indicative of a corner
+            // case where the cue was detached while we were pressing it. Let
+            // hotcueActivatePreview handle it.
+            hotcueActivatePreview(pControl, v);
+        }
+    }
+}
+
+void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
+    QMutexLocker lock(&m_mutex);
+    if (!m_pLoadedTrack)
+        return;
+    Cue* pCue = pControl->getCue();
+
+    if (v) {
+        if (pCue && pCue->getPosition() != -1) {
+            m_iCurrentlyPreviewingHotcues++;
+            int iPosition = pCue->getPosition();
+            m_pPlayButton->set(1.0);
+            m_bPreviewingHotcue = true;
+            pControl->setPreviewing(true);
+            pControl->setPreviewingPosition(iPosition);
+
+            // Need to unlock before emitting any signals to prevent deadlock.
+            lock.unlock();
+
+            emit(seekAbs(iPosition));
+        }
+    } else if (m_bPreviewingHotcue) {
+        // This is a activate release and we are previewing at least one
+        // hotcue. If this hotcue is previewing:
+        if (pControl->isPreviewing()) {
+            // Mark this hotcue as not previewing.
+            int iPosition = pControl->getPreviewingPosition();
+            pControl->setPreviewing(false);
+            pControl->setPreviewingPosition(-1);
+
+            // If this is the last hotcue to leave preview.
+            if (--m_iCurrentlyPreviewingHotcues == 0) {
+                bool bHotcueCancel = m_bHotcueCancel;
+                m_bPreviewingHotcue = false;
+                m_bHotcueCancel = false;
+                // If hotcue cancel is marked then do not snap back to the
+                // hotcue and stop. Otherwise, seek back to the start point and
+                // stop.
+                if (bHotcueCancel) {
+                    // Re-trigger the play button value so controllers get the correct one
+                    // after.
+                    m_pPlayButton->set(m_pPlayButton->get());
+                } else {
+                    m_pPlayButton->set(0.0);
+                    // Need to unlock before emitting any signals to prevent deadlock.
+                    lock.unlock();
+                    emit(seekAbs(iPosition));
+                }
+            }
+        }
+    }
+}
+
+void CueControl::hotcueClear(HotcueControl* pControl, double v) {
+    if (!v)
+        return;
+
     QMutexLocker lock(&m_mutex);
     if (!m_pLoadedTrack)
         return;
 
-    int hotcue = senderHotcue(sender());
+    Cue* pCue = pControl->getCue();
+    if (pCue) {
+        pCue->setHotCue(-1);
+    }
 
-    Cue* pCue = m_hotcue[hotcue];
+    detachCue(pControl->getHotcueNumber());
+}
+
+void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPosition) {
+    QMutexLocker lock(&m_mutex);
+    if (!m_pLoadedTrack)
+        return;
+
+    Cue* pCue = pControl->getCue();
     if (pCue) {
         // Setting the position to -1 is the same as calling hotcue_x_clear
         if (newPosition == -1) {
             pCue->setHotCue(-1);
+            detachCue(pControl->getHotcueNumber());
         } else if (newPosition > 0 && newPosition < m_pTrackSamples->get()) {
-            pCue->setPosition(newPosition);
+            int position = newPosition;
+            // People writing from MIDI land, elsewhere might be careless.
+            if (position % 2 != 0) {
+                position--;
+            }
+            pCue->setPosition(position);
         }
     }
 }
@@ -494,8 +547,10 @@ void CueControl::hintReader(QList<Hint>& hintList) {
     }
 
     for (int i = 0; i < m_iNumHotCues; ++i) {
-        if (m_hotcue[i] != NULL) {
-            double position = m_hotcuePosition[i]->get();
+        HotcueControl* pControl = m_hotcueControl[i];
+        Cue *pCue = pControl->getCue();
+        if (pCue != NULL) {
+            double position = pControl->getPosition()->get();
             if (position != -1) {
                 cue_hint.sample = position;
                 if (cue_hint.sample % 2 != 0)
@@ -519,8 +574,9 @@ void CueControl::cueSet(double v) {
         return;
 
     QMutexLocker lock(&m_mutex);
-    double cue = math_max(0.,round(getCurrentSample()));
-    if (!even((int)cue))
+    double cue = (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
+            floorf(m_pClosestBeat->get()) : floorf(getCurrentSample());
+    if (!even(cue))
         cue--;
     m_pCuePoint->set(cue);
     saveCuePoint(cue);
@@ -532,19 +588,24 @@ void CueControl::cueGoto(double v)
         return;
 
     QMutexLocker lock(&m_mutex);
-    // Set cue point if play is not pressed
+    // Seek to cue point
+    double cuePoint = m_pCuePoint->get();
+
+    // Need to unlock before emitting any signals to prevent deadlock.
+    lock.unlock();
+
+    emit(seekAbs(cuePoint));
+}
+
+void CueControl::cueGotoAndPlay(double v)
+{
+    if (!v)
+        return;
+    cueGoto(v);
+    QMutexLocker lock(&m_mutex);
+    // Start playing if not already
     if (m_pPlayButton->get()==0.) {
-        // Set the cue point and play
-        cueSet(v);
         m_pPlayButton->set(1.0);
-    } else {
-        // Seek to cue point
-        double cuePoint = m_pCuePoint->get();
-
-        // Need to unlock before emitting any signals to prevent deadlock.
-        lock.unlock();
-
-        emit(seekAbs(cuePoint));
     }
 }
 
@@ -604,10 +665,10 @@ void CueControl::cueSimple(double v) {
 
 void CueControl::cueCDJ(double v) {
     /* This is how CDJ cue buttons work:
-     * If pressed while playing, stop playback at go to cue.
+     * If pressed while playing, stop playback and go to cue.
      * If pressed while stopped and at cue, play while pressed.
      * If pressed while stopped and not at cue, set new cue point.
-     * TODO: If play is pressed while holding cue, the deck is now playing.
+     * If play is pressed while holding cue, the deck is now playing. (Handled in playFromCuePreview().)
      */
 
     QMutexLocker lock(&m_mutex);
@@ -633,6 +694,13 @@ void CueControl::cueCDJ(double v) {
                 cueSet(v);
                 // Just in case.
                 m_bPreviewing = false;
+
+                // If quantize is enabled, jump to the cue point since it's not
+                // necessarily where we currently are
+                if (m_pQuantizeEnabled->get() > 0.0) {
+                    lock.unlock();  // prevent deadlock.
+                    emit(seekAbs(m_pCuePoint->get()));
+                }
             }
         }
     } else if (m_bPreviewing) {
@@ -644,6 +712,30 @@ void CueControl::cueCDJ(double v) {
 
         emit(seekAbs(cuePoint));
     }
+    else {
+        // Re-trigger the play button value so controllers get the correct one
+        // after playFromCuePreview() changes it.
+        m_pPlayButton->set(m_pPlayButton->get());
+    }
+}
+
+void CueControl::playFromCuePreview(double v) {
+    Q_UNUSED(v);
+
+    QMutexLocker lock(&m_mutex);
+
+    if (m_bPreviewing) {
+    // we're previewing? Then stop previewing and go into normal play mode.
+        m_pPlayButton->set(1.0);
+        m_bPreviewing = false;
+    }
+
+    if (m_bPreviewingHotcue) {
+        m_pPlayButton->set(1.0);
+        m_bHotcueCancel = true;
+    }
+
+    lock.unlock();
 }
 
 void CueControl::cueDefault(double v) {
@@ -655,3 +747,105 @@ void CueControl::cueDefault(double v) {
     }
 }
 
+ConfigKey HotcueControl::keyForControl(int hotcue, QString name) {
+    ConfigKey key;
+    key.group = m_pGroup;
+    // Add one to hotcue so that we dont have a hotcue_0
+    key.item = QString("hotcue_%1_%2").arg(QString::number(hotcue+1), name);
+    return key;
+}
+
+HotcueControl::HotcueControl(const char* pGroup, int i)
+        : m_pGroup(pGroup),
+          m_iHotcueNumber(i),
+          m_pCue(NULL),
+          m_bPreviewing(false),
+          m_iPreviewingPosition(-1) {
+    m_hotcuePosition = new ControlObject(keyForControl(i, "position"));
+    connect(m_hotcuePosition, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcuePositionChanged(double)),
+            Qt::DirectConnection);
+    m_hotcuePosition->set(-1);
+
+    m_hotcueEnabled = new ControlObject(keyForControl(i, "enabled"));
+    m_hotcueEnabled->set(0);
+
+    m_hotcueSet = new ControlPushButton(keyForControl(i, "set"));
+    connect(m_hotcueSet, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueSet(double)),
+            Qt::DirectConnection);
+
+    m_hotcueGoto = new ControlPushButton(keyForControl(i, "goto"));
+    connect(m_hotcueGoto, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueGoto(double)),
+            Qt::DirectConnection);
+
+    m_hotcueGotoAndPlay = new ControlPushButton(keyForControl(i, "gotoandplay"));
+    connect(m_hotcueGotoAndPlay, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueGotoAndPlay(double)),
+            Qt::DirectConnection);
+
+    m_hotcueGotoAndStop = new ControlPushButton(keyForControl(i, "gotoandstop"));
+    connect(m_hotcueGotoAndStop, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueGotoAndStop(double)),
+            Qt::DirectConnection);
+
+    m_hotcueActivate = new ControlPushButton(keyForControl(i, "activate"));
+    connect(m_hotcueActivate, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueActivate(double)),
+            Qt::DirectConnection);
+
+    m_hotcueActivatePreview = new ControlPushButton(keyForControl(i, "activate_preview"));
+    connect(m_hotcueActivatePreview, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueActivatePreview(double)),
+            Qt::DirectConnection);
+
+    m_hotcueClear = new ControlPushButton(keyForControl(i, "clear"));
+    connect(m_hotcueClear, SIGNAL(valueChanged(double)),
+            this, SLOT(slotHotcueClear(double)),
+            Qt::DirectConnection);
+}
+
+HotcueControl::~HotcueControl() {
+    delete m_hotcuePosition;
+    delete m_hotcueEnabled;
+    delete m_hotcueSet;
+    delete m_hotcueGoto;
+    delete m_hotcueGotoAndPlay;
+    delete m_hotcueGotoAndStop;
+    delete m_hotcueActivate;
+    delete m_hotcueActivatePreview;
+    delete m_hotcueClear;
+}
+
+void HotcueControl::slotHotcueSet(double v) {
+    emit(hotcueSet(this, v));
+}
+
+void HotcueControl::slotHotcueGoto(double v) {
+    emit(hotcueGoto(this, v));
+}
+
+void HotcueControl::slotHotcueGotoAndPlay(double v) {
+    emit(hotcueGotoAndPlay(this, v));
+}
+
+void HotcueControl::slotHotcueGotoAndStop(double v) {
+    emit(hotcueGotoAndStop(this, v));
+}
+
+void HotcueControl::slotHotcueActivate(double v) {
+    emit(hotcueActivate(this, v));
+}
+
+void HotcueControl::slotHotcueActivatePreview(double v) {
+    emit(hotcueActivatePreview(this, v));
+}
+
+void HotcueControl::slotHotcueClear(double v) {
+    emit(hotcueClear(this, v));
+}
+
+void HotcueControl::slotHotcuePositionChanged(double newPosition) {
+    emit(hotcuePositionChanged(this, newPosition));
+}
