@@ -19,14 +19,20 @@
 #define ENGINEBUFFER_H
 
 #include <qapplication.h>
-#include <qmutex.h>
+#include <QMutex>
+#include <QAtomicInt>
+
 #include "defs.h"
-#include "engineobject.h"
+#include "engine/engineobject.h"
 #include "trackinfoobject.h"
 #include "configobject.h"
-// #include "monitor.h"
 #include "rotary.h"
 
+//for the writer
+#ifdef __SCALER_DEBUG__
+#include <QtCore>
+#endif
+
 class EngineControl;
 class BpmControl;
 class RateControl;
@@ -43,6 +49,7 @@ class EngineBufferScale;
 class EngineBufferScaleLinear;
 class EngineBufferScaleST;
 class EngineWorkerScheduler;
+class EngineMaster;
 
 struct Hint;
 
@@ -57,7 +64,9 @@ const int audioBeatMarkLen = 40;
 const int kiTempLength = 200000;
 
 // Rate at which the playpos slider is updated (using a sample rate of 44100 Hz):
-const int UPDATE_RATE = 5;
+const int kiUpdateRate = 10;
+// Number of kiUpdateRates that go by before we update BPM.
+const int kiBpmUpdateRate = 40 / kiUpdateRate; //about 2.5 updates per sec
 
 // End of track mode constants
 const int TRACK_END_MODE_STOP = 0;
@@ -65,10 +74,18 @@ const int TRACK_END_MODE_NEXT = 1;
 const int TRACK_END_MODE_LOOP = 2;
 const int TRACK_END_MODE_PING = 3;
 
-// Maximum number of samples used to ramp to or from zero when playback is
-// stopped or started.
-const int kiRampLength = 50;
+//vinyl status constants
+//XXX: move this to vinylcontrol.h once thread startup is moved
+const int VINYL_STATUS_DISABLED = 0;
+const int VINYL_STATUS_OK = 1;
+const int VINYL_STATUS_WARNING = 2;
+const int VINYL_STATUS_ERROR = 3;
 
+const int ENGINE_RAMP_DOWN = -1;
+const int ENGINE_RAMP_NONE = 0;
+const int ENGINE_RAMP_UP = 1;
+
+//const int kiRampLength = 3;
 
 class EngineBuffer : public EngineObject
 {
@@ -76,26 +93,21 @@ class EngineBuffer : public EngineObject
 public:
     EngineBuffer(const char *_group, ConfigObject<ConfigValue> *_config);
     ~EngineBuffer();
-    /** Reconfigures the EngineBufferScaleSRC objects with the sound scale mode written in the config database */
-    void setPitchIndpTimeStretch(bool b);
     bool getPitchIndpTimeStretch(void);
 
     void bindWorkers(EngineWorkerScheduler* pWorkerScheduler);
 
-    // Request that the EngineBuffer load a track. Since the process is
-    // asynchronous, EngineBuffer will emit a trackLoaded signal when the load
-    // has completed.
-    void loadTrack(TrackPointer pTrack);
-
     // Add an engine control to the EngineBuffer
     void addControl(EngineControl* pControl);
 
-    /** Return the current rate (not thread-safe) */
+    // Return the current rate (not thread-safe)
     double getRate();
-    /** Returns current bpm value (not thread-safe) */
+    // Returns current bpm value (not thread-safe)
     double getBpm();
+    // Returns the BPM of the loaded track (not thread-safe)
+    double getFileBpm();
     /** Sets pointer to other engine buffer/channel */
-    void setOtherEngineBuffer(EngineBuffer *);
+    void setEngineMaster(EngineMaster*);
 
     /** Reset buffer playpos and set file playpos. This must only be called
       * while holding the pause mutex */
@@ -104,40 +116,59 @@ public:
     void process(const CSAMPLE *pIn, const CSAMPLE *pOut, const int iBufferSize);
 
     const char* getGroup();
-public slots:
+    bool isTrackLoaded();
+    TrackPointer getLoadedTrack() const;
+
+    // For dependency injection of readers.
+    void setReader(CachingReader* pReader);
+
+  public slots:
     void slotControlPlay(double);
+    void slotControlPlayFromStart(double);
+    void slotControlJumpToStartAndStop(double);
+    void slotControlStop(double);
     void slotControlStart(double);
     void slotControlEnd(double);
     void slotControlSeek(double);
     void slotControlSeekAbs(double);
+    void slotControlSlip(double);
 
-signals:
+    // Request that the EngineBuffer load a track. Since the process is
+    // asynchronous, EngineBuffer will emit a trackLoaded signal when the load
+    // has completed.
+    void slotLoadTrack(TrackPointer pTrack, bool play = false);
+
+    void slotEjectTrack(double);
+
+  signals:
     void trackLoaded(TrackPointer pTrack);
     void trackLoadFailed(TrackPointer pTrack, QString reason);
-    void loadNextTrack();
+    void trackUnloaded(TrackPointer pTrack);
 
-private slots:
+  private slots:
+    void slotTrackLoading();
     void slotTrackLoaded(TrackPointer pTrack,
                          int iSampleRate, int iNumSamples);
     void slotTrackLoadFailed(TrackPointer pTrack,
                              QString reason);
 
 private:
-
-    /** Called from process() when an empty buffer, possible ramped to zero is needed */
-    void rampOut(const CSAMPLE *pOut, int iBufferSize);
+    void setPitchIndpTimeStretch(bool b);
 
     void updateIndicators(double rate, int iBufferSize);
 
-    void hintReader(const double rate,
-                    const int iSourceSamples);
+    void hintReader(const double rate);
+
+    void ejectTrack();
+
+    double fractionalPlayposFromAbsolute(double absolutePlaypos);
 
     // Lock for modifying local engine variables that are not thread safe, such
     // as m_engineControls and m_hintList
     QMutex m_engineLock;
 
     /** Holds the name of the control group */
-    const char* group;
+    const char* m_group;
     ConfigObject<ConfigValue>* m_pConfig;
 
     /** Pointer to the loop control object */
@@ -155,9 +186,6 @@ private:
         ahead */
     ReadAheadManager* m_pReadAheadManager;
 
-    /** Pointer to other EngineBuffer */
-    EngineBuffer* m_pOtherEngineBuffer;
-
     // The reader used to read audio files
     CachingReader* m_pReader;
 
@@ -165,60 +193,90 @@ private:
     QList<Hint> m_hintList;
 
     /** The current sample to play in the file. */
-    double filepos_play;
+    double m_filepos_play;
     /** Copy of rate_exchange, used to check if rate needs to be updated */
-    double rate_old;
+    double m_rate_old;
     /** Copy of length of file */
-    long int file_length_old;
+    long int m_file_length_old;
     /** Copy of file sample rate*/
-    int file_srate_old;
+    int m_file_srate_old;
     /** Mutex controlling weather the process function is in pause mode. This happens
       * during seek and loading of a new track */
-    QMutex pause;
+    QMutex m_pause;
     /** Used in update of playpos slider */
     int m_iSamplesCalculated;
+    int m_iUiSlowTick;
+
+    // The location where the track would have been had slip not been engaged
+    double m_dSlipPosition;
+    // Saved value of rate for slip mode
+    double m_dSlipRate;
+    // Slip Status
+    bool m_bSlipEnabled;
 
     ControlObject* m_pTrackSamples;
+    ControlObject* m_pTrackSampleRate;
 
-    ControlPushButton *playButton, *buttonBeatSync;
-    ControlObjectThreadMain *playButtonCOT, *m_pTrackEndCOT;
-    ControlObject *fwdButton, *backButton;
+    ControlPushButton* m_playButton;
+    ControlPushButton* m_playStartButton;
+    ControlPushButton* m_stopStartButton;
+    ControlPushButton* m_stopButton;
 
-    ControlObject *rateEngine;
-    ControlObject *m_pMasterRate;
-    ControlPushButton *wheelTouchSensor, *wheelTouchSwitch;
-    ControlPotmeter *playposSlider;
-    ControlPotmeter *visualPlaypos;
-    ControlObject *m_pSampleRate;
+    ControlObject* m_fwdButton;
+    ControlObject* m_backButton;
+    ControlPushButton* m_pSlipButton;
+    ControlObject* m_pSlipPosition;
 
-    /** Mutex used in sharing buffer and abs playpos */
-    QMutex m_qPlayposMutex;
-    /** Buffer and absolute playpos shared among threads */
-    double m_dAbsPlaypos;
+    ControlObject* m_rateEngine;
+    ControlObject* m_visualBpm;
+    ControlObject* m_pMasterRate;
+    ControlPotmeter* m_playposSlider;
+    ControlPotmeter* m_visualPlaypos;
+    ControlObject* m_pSampleRate;
+    ControlPushButton* m_pKeylock;
+
+    ControlPushButton* m_pEject;
+
+    // Whether or not to repeat the track when at the end
+    ControlPushButton* m_pRepeat;
+
+    ControlObject* m_pVinylStatus;  //Status of vinyl control
+    ControlObject* m_pVinylSeek;
 
-    /** Control used to signal when at end of file */
-    ControlObject *m_pTrackEnd, *m_pTrackEndMode;
     /** Fwd and back controls, start and end of track control */
-    ControlPushButton *startButton, *endButton;
+    ControlPushButton* m_startButton;
+    ControlPushButton* m_endButton;
 
     /** Object used to perform waveform scaling (sample rate conversion) */
-    EngineBufferScale *m_pScale;
+    EngineBufferScale* m_pScale;
     /** Object used for linear interpolation scaling of the audio */
-    EngineBufferScaleLinear *m_pScaleLinear;
+    EngineBufferScaleLinear* m_pScaleLinear;
     /** Object used for pitch-indep time stretch (key lock) scaling of the audio */
-    EngineBufferScaleST *m_pScaleST;
+    EngineBufferScaleST* m_pScaleST;
     // Indicates whether the scaler has changed since the last process()
     bool m_bScalerChanged;
 
     /** Holds the last sample value of the previous buffer. This is used when ramping to
       * zero in case of an immediate stop of the playback */
-    float m_fLastSampleValue;
+    float m_fLastSampleValue[2];
     /** Is true if the previous buffer was silent due to pausing */
     bool m_bLastBufferPaused;
+    QAtomicInt m_iTrackLoading;
+    bool m_bPlayAfterLoading;
+    float m_fRampValue;
+    int m_iRampState;
+    //int m_iRampIter;
 
-    /** Whether Pitch-Independent Time Stretch should be re-enabled when we
-        start playing post-scratch **/
-    bool m_bResetPitchIndpTimeStretch; // TODO(rryan) remove?
+    TrackPointer m_pCurrentTrack;
+#ifdef __SCALER_DEBUG__
+    QFile df;
+    QTextStream writer;
+#endif
+    CSAMPLE* m_pDitherBuffer;
+    unsigned int m_iDitherBufferReadIndex;
+    CSAMPLE* m_pCrossFadeBuffer;
+    int m_iCrossFadeSamples;
+    int m_iLastBufferSize;
 };
 
 #endif
