@@ -10,13 +10,15 @@
 #include "library/libraryfeature.h"
 #include "library/dao/playlistdao.h"
 #include "configobject.h"
+#include "treeitemmodel.h"
+#include "dlgautodj.h"
 
 class PlaylistTableModel;
 class TrackCollection;
 
 class AutoDJFeature : public LibraryFeature {
     Q_OBJECT
-    public:
+  public:
     AutoDJFeature(QObject* parent,
                   ConfigObject<ConfigValue>* pConfig,
                   TrackCollection* pTrackCollection);
@@ -25,29 +27,24 @@ class AutoDJFeature : public LibraryFeature {
     QVariant title();
     QIcon getIcon();
 
-    bool dropAccept(QUrl url);
-    bool dropAcceptChild(const QModelIndex& index, QUrl url);
+    bool dropAccept(QList<QUrl> urls,QWidget *pSource);
     bool dragMoveAccept(QUrl url);
-    bool dragMoveAcceptChild(const QModelIndex& index, QUrl url);
 
-    void bindWidget(WLibrarySidebar* sidebarWidget,
-                    WLibrary* libraryWidget,
+    void bindWidget(WLibrary* libraryWidget,
                     MixxxKeyboard* keyboard);
 
-    QAbstractItemModel* getChildModel();
+    TreeItemModel* getChildModel();
 
-public slots:
+  public slots:
     void activate();
-    void activateChild(const QModelIndex& index);
-    void onRightClick(const QPoint& globalPos);
-    void onRightClickChild(const QPoint& globalPos, QModelIndex index);
 
-private:
+  private:
     ConfigObject<ConfigValue>* m_pConfig;
     TrackCollection* m_pTrackCollection;
     PlaylistDAO& m_playlistDao;
     const static QString m_sAutoDJViewName;
-    QStringListModel m_childModel;
+    TreeItemModel m_childModel;
+    DlgAutoDJ* m_pAutoDJView;
 };
 
 
