@@ -21,23 +21,21 @@
 #include "library/promotracksfeature.h"
 #include "library/bundledsongswebview.h"
 #include "library/featuredartistswebview.h"
-#include "library/proxytrackmodel.h"
 #include "library/trackcollection.h"
 #include "library/dao/cratedao.h"
 #include "trackinfoobject.h"
 #include "defs_promo.h"
 #include "widget/wlibrary.h"
-#include "widget/wlibrarysidebar.h"
 #include "mixxxkeyboard.h"
 
 QString PromoTracksFeature::m_sPromoLocalHTMLLocation;
 QString PromoTracksFeature::m_sPromoRemoteHTMLLocation;
-#define PROMO_BUNDLE_PATH (config->getConfigPath() + "/promo/" + MIXXX_PROMO_VERSION + "/")
+#define PROMO_BUNDLE_PATH (config->getResourcePath() + "promo/" + MIXXX_PROMO_VERSION + "/")
 #define LOCAL_HTML_LOCATION (PROMO_BUNDLE_PATH + "index.html")
 
-const QString PromoTracksFeature::m_sFeaturedArtistsViewName = tr("Featured Artists");
-const QString PromoTracksFeature::m_sBundledSongsViewName = tr("Bundled Songs");
-const QString PromoTracksFeature::m_sMyDownloadsViewName = tr("My Downloads");
+const QString PromoTracksFeature::m_sFeaturedArtistsViewName = "Featured Artists";
+const QString PromoTracksFeature::m_sBundledSongsViewName = "Bundled Songs";
+const QString PromoTracksFeature::m_sMyDownloadsViewName = "My Downloads";
 
 PromoTracksFeature::PromoTracksFeature(QObject* parent,
                              ConfigObject<ConfigValue>* config,
@@ -53,7 +51,7 @@ PromoTracksFeature::PromoTracksFeature(QObject* parent,
 
     m_sPromoRemoteHTMLLocation = QString("http://promo.mixxx.org/%1/index.html").arg(MIXXX_PROMO_VERSION); //m_pConfig->getConfigPath() + "/promo/promotracks.html";
     m_sPromoLocalHTMLLocation = LOCAL_HTML_LOCATION;
-    m_sPromoAutoloadLocation = m_pConfig->getConfigPath() + "/promo/" + MIXXX_PROMO_VERSION + "/autoload.dat";
+    m_sPromoAutoloadLocation = m_pConfig->getResourcePath() + "/promo/" + MIXXX_PROMO_VERSION + "/autoload.dat";
 
     //Load the extra.dat file so we can peek at some extra information, such
     //as which songs to auto-load into Mixxx's players.
@@ -66,7 +64,7 @@ PromoTracksFeature::PromoTracksFeature(QObject* parent,
         while (!extra.atEnd())
         {
             QString trackPath = extra.readLine();
-            trackPath = m_pConfig->getConfigPath() + "/promo/" + MIXXX_PROMO_VERSION + "/" + trackPath;
+            trackPath = m_pConfig->getResourcePath() + "/promo/" + MIXXX_PROMO_VERSION + "/" + trackPath;
             QFileInfo fileInfo(trackPath);
             trackPath = fileInfo.absoluteFilePath();
             //qDebug() << "PROMO: Auto-loading track" << trackPath;
@@ -93,19 +91,19 @@ PromoTracksFeature::PromoTracksFeature(QObject* parent,
     //If we've bundled songs with Mixxx, show the fancy bundled songs view
     if (QFile::exists(LOCAL_HTML_LOCATION)) {
         qDebug() << "Bundled tracks found at:" << LOCAL_HTML_LOCATION;
-        childrenStringList << m_sBundledSongsViewName;
+        childrenStringList << tr(m_sBundledSongsViewName.toUtf8().constData());
     }
     else {
         qDebug() << "No bundled tracks found, disabling view. Looked in:" << LOCAL_HTML_LOCATION;
     }
 
     QStringList childrenStringList;
-    childrenStringList <<  m_sMyDownloadsViewName;
+    childrenStringList <<  tr(m_sMyDownloadsViewName.toUtf8().constData());
     m_childModel.setStringList(childrenStringList);
 
     CrateDAO& crateDAO = pTrackCollection->getCrateDAO();
-    crateDAO.createCrate(m_sMyDownloadsViewName); //XXX: hidden = false for debug
-    m_downloadsTableModel.setTable(m_sMyDownloadsViewName);
+    crateDAO.createCrate(tr(m_sMyDownloadsViewName.toUtf8().constData())); //XXX: hidden = false for debug
+    m_downloadsTableModel.setTable(tr(m_sMyDownloadsViewName.toUtf8().constData()));
     */
 
 }
@@ -114,7 +112,7 @@ PromoTracksFeature::~PromoTracksFeature() {
 }
 
 QVariant PromoTracksFeature::title() {
-    return m_sFeaturedArtistsViewName;
+    return tr(m_sFeaturedArtistsViewName.toUtf8().constData());
 }
 
 QIcon PromoTracksFeature::getIcon() {
@@ -132,36 +130,34 @@ QList<TrackPointer> PromoTracksFeature::getTracksToAutoLoad()
     return m_tracksToAutoLoad;
 }
 
-void PromoTracksFeature::bindWidget(WLibrarySidebar* sidebarWidget,
-                                    WLibrary* libraryWidget,
+void PromoTracksFeature::bindWidget(WLibrary* libraryWidget,
                                     MixxxKeyboard* keyboard) {
-
     QString libraryPath = m_pConfig->getValueString(ConfigKey("[Playlist]","Directory"));
 
     ConfigObject<ConfigValue>* config = m_pConfig; //Long story, macros macros macros
     m_pBundledSongsView = new BundledSongsWebView(libraryWidget, m_pTrackCollection,
-                                                  PROMO_BUNDLE_PATH, 
-                                                  m_sPromoLocalHTMLLocation, 
+                                                  PROMO_BUNDLE_PATH,
+                                                  m_sPromoLocalHTMLLocation,
                                                   m_bFirstRun, m_pConfig);
     m_pBundledSongsView->installEventFilter(keyboard);
 
-    libraryWidget->registerView(m_sBundledSongsViewName, m_pBundledSongsView);
+    libraryWidget->registerView(tr(m_sBundledSongsViewName.toUtf8().constData()), m_pBundledSongsView);
     connect(m_pBundledSongsView, SIGNAL(loadTrack(TrackPointer)),
             this, SIGNAL(loadTrack(TrackPointer)));
-    connect(m_pBundledSongsView, SIGNAL(loadTrackToPlayer(TrackPointer, int)),
-            this, SIGNAL(loadTrackToPlayer(TrackPointer, int)));
+    connect(m_pBundledSongsView, SIGNAL(loadTrackToPlayer(TrackPointer, QString)),
+            this, SIGNAL(loadTrackToPlayer(TrackPointer, QString)));
 
 /*  XXX: Re-enable this code for Promo 3.0
     m_pFeaturedArtistsView = new FeaturedArtistsWebView(libraryWidget, libraryPath, m_sPromoRemoteHTMLLocation, new SongDownloader(this));
-    libraryWidget->registerView(m_sFeaturedArtistsViewName, m_pFeaturedArtistsView);
+    libraryWidget->registerView(tr(m_sFeaturedArtistsViewName.toUtf8().constData()), m_pFeaturedArtistsView);
     connect(m_pFeaturedArtistsView, SIGNAL(loadTrack(TrackInfoObject*)),
             this, SIGNAL(loadTrack(TrackInfoObject*)));
-    connect(m_pFeaturedArtistsView, SIGNAL(loadTrackToPlayer(TrackInfoObject*, int)),
-            this, SIGNAL(loadTrackToPlayer(TrackInfoObject*, int)));
+    connect(m_pFeaturedArtistsView, SIGNAL(loadTrackToPlayer(TrackInfoObject*, QString)),
+            this, SIGNAL(loadTrackToPlayer(TrackInfoObject*, QString)));
     */
 }
 
-QAbstractItemModel* PromoTracksFeature::getChildModel() {
+TreeItemModel* PromoTracksFeature::getChildModel() {
     //XXX Promo 3.0:
     //return NULL;
     return &m_childModel;
@@ -169,40 +165,15 @@ QAbstractItemModel* PromoTracksFeature::getChildModel() {
 
 void PromoTracksFeature::activate() {
     //XXX Promo 3.0:
-    //emit(switchToView(m_sFeaturedArtistsViewName));
-    emit(switchToView(m_sBundledSongsViewName));
+    //emit(switchToView(tr(m_sFeaturedArtistsViewName.toUtf8().constData())));
+    emit(switchToView(tr(m_sBundledSongsViewName.toUtf8().constData())));
 }
 
 void PromoTracksFeature::activateChild(const QModelIndex& index) {
     QString itemString = m_childModel.data(index, Qt::DisplayRole).toString();
-    if (itemString == m_sMyDownloadsViewName)
-    {
+    if (itemString == tr(m_sMyDownloadsViewName.toUtf8().constData())) {
         emit(showTrackModel(&m_downloadsTableModel));
-    }
-    else
+    } else {
         emit(switchToView(itemString));
-}
-
-void PromoTracksFeature::onRightClick(const QPoint& globalPos) {
-}
-
-void PromoTracksFeature::onRightClickChild(const QPoint& globalPos,
-                                            QModelIndex index) {
-}
-
-bool PromoTracksFeature::dropAccept(QUrl url) {
-    return false;
-}
-
-bool PromoTracksFeature::dropAcceptChild(const QModelIndex& index, QUrl url) {
-    return false;
-}
-
-bool PromoTracksFeature::dragMoveAccept(QUrl url) {
-    return false;
-}
-
-bool PromoTracksFeature::dragMoveAcceptChild(const QModelIndex& index,
-                                              QUrl url) {
-    return false;
+    }
 }
