@@ -23,7 +23,6 @@
 #include <math.h>
 #include <algorithm>
 
-static CSAMPLE two_pi = (2.f*acos(-1.f));
 
 CSAMPLE besseli(CSAMPLE);
 int sign(CSAMPLE);
@@ -41,7 +40,22 @@ bool even(long n);
 /** Compute pow(x,n) for positive integer n through repeated
   * squarings */
 double qip(CSAMPLE x, unsigned int n);
+float sigmoid_zero(double t, double max_t);
+
+static const CSAMPLE pi     = acos(-1.0f);
+static const CSAMPLE two_pi = (2.f*acos(-1.f));
+
+int nearestSuperiorPowerOfTwo(int v);
+
+#ifdef _MSC_VER
+// VC++ uses _isnan() instead of isnan()
+#include <float.h>
+#define isnan(x) _isnan(x)
+#else
+// for isnan() everywhere else
+#include <cmath>
+using std::isnan;
+#endif
 
-static CSAMPLE pi     = acos(-1.0f);
 
 #endif
