@@ -19,24 +19,49 @@
 
 #include <QObject>
 #include <QMutex>
+#include <QMap>
+#include <QTimerEvent>
+
+class ControlObjectThread;
 
 #include "trackinfoobject.h"
 
 class PlayerInfo : public QObject
 {
     Q_OBJECT
-public:
+  public:
     static PlayerInfo &Instance();
-    TrackPointer getTrackInfo(int track);
-    void setTrackInfo(int track, TrackPointer trackInfoObj);
-private:
+    TrackPointer getTrackInfo(QString group);
+    void setTrackInfo(QString group, TrackPointer trackInfoObj);
+    int getCurrentPlayingDeck();
+    TrackPointer getCurrentPlayingTrack();
+    bool isTrackLoaded(TrackPointer pTrack) const;
+    bool isTrackPlaying(TrackPointer pTrack) const;
+
+  signals:
+    void currentPlayingDeckChanged(int deck);
+    void trackLoaded(QString group, TrackPointer pTrack);
+    void trackUnloaded(QString group, TrackPointer pTrack);
+
+  private:
+    void timerEvent(QTimerEvent* pTimerEvent);
+    void updateCurrentPlayingDeck();
+
     PlayerInfo();
     ~PlayerInfo();
     PlayerInfo(PlayerInfo const&);
     PlayerInfo &operator= (PlayerInfo const&);
-    QMutex m_mutex;
-    TrackPointer m_pTrack1;
-    TrackPointer m_pTrack2;
+    mutable QMutex m_mutex;
+    int m_iNumDecks;
+    int m_iNumSamplers;
+    ControlObjectThread* m_COxfader;
+    QMap<QString, TrackPointer> m_loadedTrackMap;
+    QMap<QString, ControlObjectThread*> m_listCOPlay;
+    QMap<QString, ControlObjectThread*> m_listCOVolume;
+    QMap<QString, ControlObjectThread*> m_listCOOrientation;
+    QMap<QString, ControlObjectThread*> m_listCOpregain;
+
+    int m_currentlyPlayingDeck;
 };
 
 #endif
