@@ -14,12 +14,15 @@
 *                                                                         *
 ***************************************************************************/
 
+#include <taglib/mpegfile.h>
+
 #include "soundsourcemp3.h"
 #include <QtDebug>
 
 
-SoundSourceMp3::SoundSourceMp3(QString qFilename) : SoundSource(qFilename),
-                                                    m_file(qFilename)
+SoundSourceMp3::SoundSourceMp3(QString qFilename) :
+        Mixxx::SoundSource(qFilename),
+        m_file(qFilename)
 {
     inputbuf = NULL;
     Stream = new mad_stream;
@@ -28,19 +31,34 @@ SoundSourceMp3::SoundSourceMp3(QString qFilename) : SoundSource(qFilename),
     mad_synth_init(Synth);
     Frame = new mad_frame;
     mad_frame_init(Frame);
+
+    m_currentSeekFrameIndex = 0;
+    m_iAvgFrameSize = 0;
+    m_iChannels = 0;
+    rest = 0;
 }
 
 SoundSourceMp3::~SoundSourceMp3()
 {
     mad_stream_finish(Stream);
+    delete Stream;
+
     mad_frame_finish(Frame);
+    delete Frame;
+
     mad_synth_finish(Synth);
+    delete Synth;
+
+    // Unmap inputbuf.
     m_file.unmap(inputbuf);
+    inputbuf = NULL;
     m_file.close();
 
-    //Don't delete because we're using mmapped i/o now.... right?
-    //delete [] inputbuf;
-
+    //Free the pointers in our seek list, LIBERATE THEM!!!
+    for (int i = 0; i < m_qSeekList.count(); i++)
+    {
+        delete m_qSeekList[i];
+    }
     m_qSeekList.clear();
 }
 
@@ -103,12 +121,16 @@ int SoundSourceMp3::open()
 
         // Grab data from Header
 
-        // This warns us only when the reported sample rate changes. (and when it is first set)
-        if(m_iSampleRate != Header.samplerate) {
-            //qDebug() << "SSMP3() :: Setting m_iSampleRate to " << Header.samplerate << " from " << m_iSampleRate;
+        // This warns us only when the reported sample rate changes. (and when
+        // it is first set)
+        if (m_iSampleRate == 0 && Header.samplerate > 0) {
+            setSampleRate(Header.samplerate);
+        } else if (m_iSampleRate != Header.samplerate) {
+            qDebug() << "SSMP3: file has differing samplerate in some headers:"
+                     << m_qFilename
+                     << m_iSampleRate << "vs" << Header.samplerate;
         }
 
-        setSampleRate(Header.samplerate);
         m_iChannels = MAD_NCHANNELS(&Header);
         mad_timer_add (&filelength, Header.duration);
         bitrate += Header.bitrate;
@@ -118,24 +140,21 @@ int SoundSourceMp3::open()
         p->m_pStreamPos = (unsigned char *)Stream->this_frame;
         p->pos = length();
         m_qSeekList.append(p);
-
         currentframe++;
     }
     //qDebug() << "channels " << m_iChannels;
 
-    // Find average frame size
-    if(currentframe)
-        m_iAvgFrameSize = length()/currentframe;
-    else
-        m_iAvgFrameSize = 0;
-
     mad_header_finish (&Header); // This is a macro for nothing.
 
-    if (currentframe==0)
-        bitrate = 0;
-    else
-        bitrate = bitrate/currentframe;
+    // This is not a working MP3 file.
+    if (currentframe == 0) {
+        return ERR;
+    }
 
+    // Find average frame size
+    m_iAvgFrameSize = (currentframe == 0) ? 0 : length()/currentframe;
+    // And average bitrate
+    bitrate = (currentframe == 0) ? 0 : bitrate / currentframe;
     framecount = currentframe;
     currentframe = 0;
 
@@ -160,22 +179,37 @@ int SoundSourceMp3::open()
     qDebug() << "Size    = " << length();
  */
 
-    m_qSeekList.setAutoDelete(true);
-
     // Re-init buffer:
     seek(0);
 
     return OK;
 }
 
-long SoundSourceMp3::seek(long filepos)
-{
+bool SoundSourceMp3::isValid() const {
+    return framecount > 0;
+}
+
+MadSeekFrameType* SoundSourceMp3::getSeekFrame(long frameIndex) const {
+    if (frameIndex < 0 || frameIndex >= m_qSeekList.size()) {
+        return NULL;
+    }
+    return m_qSeekList.at(frameIndex);
+}
+
+long SoundSourceMp3::seek(long filepos) {
     // Ensure that we are seeking to an even filepos
-    Q_ASSERT(filepos%2==0);
+    if (filepos % 2 != 0) {
+        qDebug() << "SoundSourceMp3 got non-even seek target.";
+        filepos--;
+    }
+
+    if (!isValid()) {
+        return 0;
+    }
 
 //     qDebug() << "SEEK " << filepos;
 
-    MadSeekFrameType * cur;
+    MadSeekFrameType* cur = NULL;
 
     if (filepos==0)
     {
@@ -189,7 +223,10 @@ long SoundSourceMp3::seek(long filepos)
         mad_frame_init(Frame);
         mad_synth_init(Synth);
         rest=-1;
-        cur = m_qSeekList.at(0);
+
+        m_currentSeekFrameIndex = 0;
+        cur = getSeekFrame(0);
+        //frameIterator.toFront(); //Might not need to do this -- Albert June 19/2010 (during Qt3 purge)
     }
     else
     {
@@ -208,10 +245,7 @@ long SoundSourceMp3::seek(long filepos)
          */
 
         int framePos = findFrame(filepos);
-
-//         qDebug() << "list length " << m_qSeekList.count();
-
-        if (framePos==0 || framePos>filepos || m_qSeekList.at()<5)
+        if (framePos==0 || framePos>filepos || m_currentSeekFrameIndex < 5)
         {
             //qDebug() << "Problem finding good seek frame (wanted " << filepos << ", got " << framePos << "), starting from 0";
 
@@ -223,50 +257,49 @@ long SoundSourceMp3::seek(long filepos)
             mad_frame_init(Frame);
             mad_synth_init(Synth);
             rest = -1;
-            cur = m_qSeekList.first();
+            m_currentSeekFrameIndex = 0;
+            cur = getSeekFrame(m_currentSeekFrameIndex);
         }
         else
         {
 //             qDebug() << "frame pos " << cur->pos;
 
             // Start four frame before wanted frame to get in sync...
-            m_qSeekList.prev();
-            m_qSeekList.prev();
-            m_qSeekList.prev();
-            cur = m_qSeekList.prev();
+            m_currentSeekFrameIndex -= 4;
+            cur = getSeekFrame(m_currentSeekFrameIndex);
+            if (cur != NULL) {
+                // Start from the new frame
+                mad_stream_finish(Stream);
+                mad_stream_init(Stream);
+                mad_stream_options(Stream, MAD_OPTION_IGNORECRC);
+                //        qDebug() << "mp3 restore " << cur->m_pStreamPos;
+                mad_stream_buffer(Stream, (const unsigned char *)cur->m_pStreamPos,
+                                  inputbuf_len-(long int)(cur->m_pStreamPos-(unsigned char *)inputbuf));
 
-            // Start from the new frame
-            mad_stream_finish(Stream);
-            mad_stream_init(Stream);
-            mad_stream_options(Stream, MAD_OPTION_IGNORECRC);
-//        qDebug() << "mp3 restore " << cur->m_pStreamPos;
-            mad_stream_buffer(Stream, (const unsigned char *)cur->m_pStreamPos, inputbuf_len-(long int)(cur->m_pStreamPos-(unsigned char *)inputbuf));
+                // Mute'ing is done here to eliminate potential pops/clicks from skipping
+                // Rob Leslie explains why here:
+                // http://www.mars.org/mailman/public/mad-dev/2001-August/000321.html
+                mad_synth_mute(Synth);
+                mad_frame_mute(Frame);
 
-            // Mute'ing is done here to eliminate potential pops/clicks from skipping
-            // Rob Leslie explains why here:
-            // http://www.mars.org/mailman/public/mad-dev/2001-August/000321.html
-            mad_synth_mute(Synth);
-            mad_frame_mute(Frame);
+                // Decode the three frames before
+                mad_frame_decode(Frame,Stream);
+                mad_frame_decode(Frame,Stream);
+                mad_frame_decode(Frame,Stream);
+                mad_frame_decode(Frame,Stream);
 
-            // Decode the three frames before
-            mad_frame_decode(Frame,Stream);
-            mad_frame_decode(Frame,Stream);
-            mad_frame_decode(Frame,Stream);
-            mad_frame_decode(Frame,Stream);
+                // this is also explained in the above mad-dev post
+                mad_synth_frame(Synth, Frame);
 
-            // this is also explained in the above mad-dev post
-            mad_synth_frame(Synth, Frame);
-
-            // Set current position
-            rest = -1;
-            m_qSeekList.next();
-            m_qSeekList.next();
-            m_qSeekList.next();
-            cur = m_qSeekList.next();
+                // Set current position
+                rest = -1;
+                m_currentSeekFrameIndex += 4;
+                cur = getSeekFrame(m_currentSeekFrameIndex);
+            }
         }
 
         // Synthesize the the samples from the frame which should be discard to reach the requested position
-        if (cur) //the "if" prevents crashes on bad files.
+        if (cur != NULL) //the "if" prevents crashes on bad files.
             discard(filepos-cur->pos);
     }
 /*
@@ -317,8 +350,7 @@ long SoundSourceMp3::seek(long filepos)
 
 }
 
-inline long unsigned SoundSourceMp3::length()
-{
+inline long unsigned SoundSourceMp3::length() {
     enum mad_units units;
 
     switch (m_iSampleRate)
@@ -405,9 +437,15 @@ unsigned long SoundSourceMp3::discard(unsigned long samples_wanted)
  */
 unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _destination)
 {
+    if (!isValid())
+        return 0;
+
     // Ensure that we are reading an even number of samples. Otherwise this function may
     // go into an infinite loop
-    Q_ASSERT(samples_wanted%2==0);
+    if (samples_wanted % 2 != 0) {
+        qDebug() << "SoundSourceMp3 got non-even samples_wanted";
+        samples_wanted--;
+    }
 //     qDebug() << "frame list " << m_qSeekList.count();
 
     SAMPLE * destination = (SAMPLE *)_destination;
@@ -431,7 +469,7 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
             else
                 *(destination++) = madScale(Synth->pcm.samples[0][i]);
 
-            // This is safe because we have Q_ASSERTed that samples_wanted is even.
+            // This is safe because we have checked that samples_wanted is even.
             Total_samples_decoded += 2;
 
         }
@@ -524,290 +562,47 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
 
 int SoundSourceMp3::parseHeader()
 {
-    QString location = m_qFilename;
-
     setType("mp3");
+#ifdef __WINDOWS__
+    /* From Tobias: A Utf-8 string did not work on my Windows XP (German edition)
+     * If you try this conversion, f.isValid() will return false in many cases
+     * and processTaglibFile() will fail
+     *
+     * The method toLocal8Bit() returns the local 8-bit representation of the string as a QByteArray.
+     * The returned byte array is undefined if the string contains characters not supported
+     * by the local 8-bit encoding.
+     */
+    QByteArray qBAFilename = m_qFilename.toLocal8Bit();
+#else
+    QByteArray qBAFilename = m_qFilename.toUtf8();
+#endif
+    TagLib::MPEG::File f(qBAFilename.constData());
 
-    // mad-dev post in 2002-25-Jan on how to use libid3 by Rob Leslie
-    // http://www.mars.org/mailman/public/mad-dev/2002-January/000439.html
-    id3_file * fh = id3_file_open(qstrdup(location.toLocal8Bit()), ID3_FILE_MODE_READONLY);
-    if (!fh) {
-		qDebug() << "SSMP3::ParseHeader : error opening ID3 tags";
-		//return ERR; //the file can still be valid without ID3 tags...
-	} else {
-        id3_tag * tag = id3_file_tag(fh);
-        if (tag!=0) {
-            // Frame names can be found here:
-            // http://www.id3.org/id3v2.4.0-frames.txt
-            QString s;
-            getField(tag,"TIT2",&s); // TIT2 : Title
-            if (s != "")
-                setTitle(s);
-            s="";
-            getField(tag,"TPE1",&s); // TPE1 : Artist
-            if (s != "")
-                setArtist(s);
-            s="";
-            getField(tag,"TALB",&s);
-            setAlbum(s);
-            getField(tag,"TDRC",&s);
-            setYear(s);
-            s="";
-            getField(tag,"TCON",&s);
-            setGenre(s);
-            s="";
-            getField(tag,"TRCK",&s);
-            setTrackNumber(s);
-            s="";
-            getField(tag,"TBPM",&s); // TBPM: the bpm
-            float bpm = 0;
-            if (s.length()>1) bpm = str2bpm(s);
-            if(bpm > 0) {
-                setBPM(bpm);
-                //Track->setBpmConfirm(true);
-            }
-            s="";
-            getField(tag, "COMM", &s);
-            setComment(s);
-            //Track->setHeaderParsed(true);
+    // Takes care of all the default metadata
+    bool result = processTaglibFile(f);
 
-            /*
-               // On some tracks this segfaults. TLEN is very seldom used anyway...
-               QString dur;
-               getField(tag,"TLEN",&dur);
-               if (dur.length()>0)
-                Track->m_iDuration = dur.toInt();
-             */
-        }
-        // this closes 'tag' for us
-        id3_file_close(fh);
+    // Now look for MP3 specific metadata (e.g. BPM)
+    TagLib::ID3v2::Tag* id3v2 = f.ID3v2Tag();
+    if (id3v2) {
+        processID3v2Tag(id3v2);
     }
 
-
-    // Get file length. This has to be done by one of these options:
-    // 1) looking for the tag named TLEN (above),
-    //   -- too buggy to rely on
-    // 2) See if the first frame contains a Xing header to get frame count
-    //   -- Xing is non-Free, so we can't
-    // 3) Find out if it is a variable frame size file by looking at the
-    //    size of the first n frames. If constant size, estimate frame number
-    //    from one frame size and file length in bytes
-    //   -- We do this, 7/2008
-    // 4) Count all the frames (slooow)
-    //   -- We do not do this, 7/2008
-
-    // Open file, initialize MAD and read beginnning of file
-
-    QFile file(location);
-    if (!file.open(QIODevice::ReadOnly)) {
-        qDebug() << "SSMP3::ParseHeader Open failed:" << location;
-        return ERR;
-    }
-    inputbuf = new unsigned char[READLENGTH];
-    /*
-    inputbuf_len = file.size();
-    inputbuf = file.map(0, inputbuf_len);
-    */
-
-    mad_header Header;
-
-    mad_stream_init(Stream);
-    mad_stream_options(Stream, MAD_OPTION_IGNORECRC);
-    mad_header_init(&Header);
-
-    // Stats we record as we go
-    mad_timer_t dur = mad_timer_zero;
-    unsigned long bitrate = 0;
-    unsigned long bytesperframe = 0;
-    bool constantbitrate = true;
-    unsigned int frames = 0;
-    unsigned long averageBitrate = 0;
-    unsigned long averageBytesPerFrame = 0;
-
-    // Number of frames to read to estimate the duration
-    const unsigned int DESIRED_FRAMES = 10;
-
-    while(frames < DESIRED_FRAMES) {
-        unsigned int readbytes = file.read((char*)inputbuf, READLENGTH);
-
-        if(readbytes != READLENGTH) {
-            //qDebug() << "MAD: ERR reading mp3-file:" << location << "\nRead only" << readbytes << "bytes, but wanted" << READLENGTH << "bytes";
-            if(readbytes == -1) {
-                // fatal error, no bytes were read
-                qDebug() << "MAD: fatal error reading mp3 file";
-                return ERR;
-            } else if(readbytes == 0) {
-                // EOF, just break out of the loop
-                break;
-            }
-            // otherwise we just have less data to work with, keep going!
-        }
-
-        // This preserves skiplen, so if we had a buffer error earlier
-        // the skip will occur when we give it more data.
-        mad_stream_buffer(Stream, (unsigned char *) inputbuf, readbytes);
-
-        while((Stream->bufend - Stream->this_frame) > 0) {
-            if(mad_header_decode(&Header,Stream) == -1) {
-                if(!MAD_RECOVERABLE(Stream->error)) {
-                    if(Stream->error == MAD_ERROR_BUFLEN) {
-                        // We skipped too much, that's ok.
-                        break;
-                    }
-                    // Something worse happened
-                    //qDebug() << "SSMP3::ParseHeaeder - MAD unrecoverable error: " << mad_stream_errorstr(Stream);
-                    break;
-                } else if(Stream->error == MAD_ERROR_LOSTSYNC) {
-                    // ignore LOSTSYNC due to ID3 tags
-                    int tagsize = id3_tag_query(Stream->this_frame, Stream->bufend - Stream->this_frame);
-                    if (tagsize > 0) {
-                        //qDebug() << "SSMP3::ParseHeader Skipping ID3 tag size: " << tagsize;
-                        mad_stream_skip(Stream, tagsize);
-                        continue;
-                    }
-                }
-                continue;
-            }
-            if (frames==0) {
-                bitrate = Header.bitrate;
-                averageBitrate += bitrate;
-                dur = Header.duration;
-                bytesperframe = (Stream->next_frame - Stream->this_frame);
-                //Set samplerate and channels here so length() calculation
-                //works.
-                setSampleRate(Header.samplerate);
-                setChannels(MAD_NCHANNELS(&Header));
-            } else if (bitrate != Header.bitrate) {
-                bytesperframe = (Stream->next_frame - Stream->this_frame);
-                constantbitrate = false;
-                averageBitrate += bitrate;
-                averageBytesPerFrame += bytesperframe;
-                //Testing to see if we can get better averages by sampling
-                //frames throughout the song, rather than all at the start.
-                //(Answer: No. I think Stream->next_frame - Stream->this_frame
-                // is an incorrect way of getting the frame size (!). XXX
-                //mad_stream_skip(Stream, 1024*64); //Skip 64kb forwards in the file
-            }
-            frames++;
-        }
+    TagLib::APE::Tag *ape = f.APETag();
+    if (ape) {
+        processAPETag(ape);
     }
 
-    //Calculate the average bitrate over a bunch of frames so we can guess the duration of VBR MP3s better
-    if (frames > 0) { //divide by zero protection
-        averageBitrate = averageBitrate / frames;
-        averageBytesPerFrame = averageBytesPerFrame / frames;
-    }
-
-    /*
-    qDebug() << "SSMP3::ParseHeader -" << getTitle();
-    qDebug() << "SSMP3::ParseHeader - frames read: " << frames << " bitrate " << Header.bitrate/1000;
-    qDebug() << "SSMP3::ParseHeader - samplerate " << Header.samplerate << " channels " << MAD_NCHANNELS(&Header);
-    */
-
-    int duration = -1;   // This default will cause the library to display a '?'
-    if (constantbitrate && frames>0) {
-        // This means that duration is an approximation.
-        // We take the duration of one frame and multiply it by
-        // the number of frames we think can fit in the file size.
-
-        // duration per frame * file_length bytes / (bytes per frame) = duration
-        //mad_timer_multiply(&dur, Track->getLength()/((Stream->this_frame-Stream->buffer)/frames));
-        if (bytesperframe > 0) //prevent div by zero
-            mad_timer_multiply(&dur, file.size()/bytesperframe);
-        duration = mad_timer_count(dur, MAD_UNITS_SECONDS);
-        setBitrate(Header.bitrate/1000);
-        //qDebug() << "SSMP3::ParseHeader - Song is CBR";
-     }
-     else //Calculate duration for VBR MP3s
-     {
-         // FIXME: We need a fast way to get reasonable VBR MP3 file duration estimates
-
-         // In the meantime (and since the below method can be waaaay off)
-         // we'd rather show nothing until the track is loaded and the real
-         // duration can be calculated - Sean (June 2010)
-
-         /*
-         //Since this is sort of a crapshoot to begin with unless you want to
-         //get really fancy, we're going to just use our estimate at the
-         //average bytes per frame to estimate the duration.
-         if (averageBytesPerFrame > 0)  //no divide by zero for you!
-             mad_timer_multiply(&dur, file.size()/averageBytesPerFrame);
-         duration = mad_timer_count(dur, MAD_UNITS_SECONDS);
-         */
-
-         setBitrate(averageBitrate/1000);
-         //qDebug() << "SSMP3::ParseHeader - Song is VBR";
-     }
-
-    setDuration(duration);
-    setSampleRate(Header.samplerate);
-    setChannels(MAD_NCHANNELS(&Header));
-
-    mad_stream_finish(Stream);
-    delete [] inputbuf;
-    inputbuf = NULL;
-    file.close();
-
-    //file.unmap(inputbuf);
-    //file.close();
-    /**
-      This code is now called in the destructor since ParseHeader() isn't
-      static any more!
-    mad_stream_finish(Stream);
-    delete [] inputbuf;
-    **/
-    return OK;
-
-}
-
-void SoundSourceMp3::getField(id3_tag * tag, const char * frameid, QString * str)
-{
-    // fetches the 0th frame from tag with frameid
-    id3_frame * frame = id3_tag_findframe(tag, frameid, 0);
-    if (frame)
-    {
-        id3_utf16_t* framestr = NULL;
-
-        // Unicode handling of various fields
-
-        if (strcmp(frameid, "COMM") == 0) {
-            id3_ucs4_t const* comment = id3_field_getfullstring(&frame->fields[3]);
-            if (!comment)
-                return;
-            framestr = id3_ucs4_utf16duplicate(comment);
-        }
-        else if (id3_field_getnstrings(&frame->fields[1])>0)
-        {
-            //Shitty genre tag handling, thanks libid3tag!
-            if (strcmp(frameid, "TCON") == 0)
-            {
-                id3_ucs4_t const* genre = id3_genre_name(id3_field_getstrings(&frame->fields[1], 0));
-                framestr = id3_ucs4_utf16duplicate(genre);
-            }
-            else
-                framestr = id3_ucs4_utf16duplicate(id3_field_getstrings(&frame->fields[1], 0));
-        }
-
-        if (!framestr)
-            return;
-
-        int strlen = 0; while (framestr[strlen]!=0) strlen++;
-        if (strlen>0) {
-            str->setUtf16((ushort *)framestr,strlen);
-            //The ID3 specification says that a tag can contain a UTF-16 byte-order-mark (BOM). If we don't
-            //remove these by hand, they will end up in strange places like our library XML file and
-            //break it. :/ Conclusion: libid3tag sucks.
-            *str = str->remove(QChar(QChar::ByteOrderMark));
-            *str = str->remove(QChar(QChar::ByteOrderSwapped));
-        }
-        free(framestr);
-    }
+    if (result)
+        return OK;
+    return ERR;
 }
 
 int SoundSourceMp3::findFrame(int pos)
 {
     // Guess position of frame in m_qSeekList based on average frame size
-    MadSeekFrameType * temp = m_qSeekList.at(math_min(m_qSeekList.count()-1, m_iAvgFrameSize ? (unsigned int)(pos/m_iAvgFrameSize) : 0));
+    m_currentSeekFrameIndex = math_min(m_qSeekList.count()-1,
+                                       m_iAvgFrameSize ? (unsigned int)(pos/m_iAvgFrameSize) : 0);
+    MadSeekFrameType* temp = getSeekFrame(m_currentSeekFrameIndex);
 
 /*
     if (temp!=0)
@@ -817,29 +612,33 @@ int SoundSourceMp3::findFrame(int pos)
  */
 
     // Ensure that the list element is not at a greater position than pos
-    while (temp!=0 && temp->pos>pos)
+    while (temp != NULL && temp->pos > pos)
     {
-        temp = m_qSeekList.prev();
+        m_currentSeekFrameIndex--;
+        temp = getSeekFrame(m_currentSeekFrameIndex);
 //        if (temp!=0) qDebug() << "backing " << pos << ", got " << temp->pos;
     }
 
     // Ensure that the following position is also not smaller than pos
-    if (temp!=0)
+    if (temp != NULL)
     {
-        temp = m_qSeekList.current();
-        while (temp!=0 && temp->pos<pos)
+        temp = getSeekFrame(m_currentSeekFrameIndex);
+        while (temp != NULL && temp->pos < pos)
         {
-            temp = m_qSeekList.next();
+            m_currentSeekFrameIndex++;
+            temp = getSeekFrame(m_currentSeekFrameIndex);
 //            if (temp!=0) qDebug() << "fwd'ing " << pos << ", got " << temp->pos;
         }
 
-        if (temp==0)
-            temp = m_qSeekList.last();
+        if (temp == NULL)
+            m_currentSeekFrameIndex = m_qSeekList.count()-1;
         else
-            temp = m_qSeekList.prev();
+            m_currentSeekFrameIndex--;
+
+        temp = getSeekFrame(m_currentSeekFrameIndex);
     }
 
-    if (temp>0)
+    if (temp != NULL)
     {
 //        qDebug() << "ended at " << pos << ", got " << temp->pos;
         return temp->pos;
