@@ -28,26 +28,38 @@
 #include "soundsourceproxy.h"
 #include "xmlparse.h"
 #include "controlobject.h"
+#include "waveform/waveform.h"
+#include "track/beatfactory.h"
 
 #include "mixxxutils.cpp"
 
 TrackInfoObject::TrackInfoObject(const QString sLocation, bool parseHeader)
-        : m_qMutex(QMutex::Recursive) {
+        : m_qMutex(QMutex::Recursive),
+          m_waveform(new Waveform()),
+          m_waveformSummary(new Waveform()),
+          m_analyserProgress(-1) {
     QFileInfo fileInfo(sLocation);
     populateLocation(fileInfo);
     initialize(parseHeader);
 }
 
-TrackInfoObject::TrackInfoObject(QFileInfo& fileInfo, bool parseHeader)
-        : m_qMutex(QMutex::Recursive) {
+TrackInfoObject::TrackInfoObject(const QFileInfo& fileInfo, bool parseHeader)
+        : m_qMutex(QMutex::Recursive),
+          m_waveform(new Waveform()),
+          m_waveformSummary(new Waveform()),
+          m_analyserProgress(-1) {
     populateLocation(fileInfo);
     initialize(parseHeader);
 }
 
 TrackInfoObject::TrackInfoObject(const QDomNode &nodeHeader)
-        : m_qMutex(QMutex::Recursive) {
+        : m_qMutex(QMutex::Recursive),
+          m_waveform(new Waveform()),
+          m_waveformSummary(new Waveform()),
+          m_analyserProgress(-1) {
     m_sFilename = XmlParse::selectNodeQString(nodeHeader, "Filename");
     m_sLocation = XmlParse::selectNodeQString(nodeHeader, "Filepath") + "/" +  m_sFilename;
+    QString create_date;
 
     // We don't call initialize() here because it would end up calling parse()
     // on the file. Plus those initializations weren't done before, so it might
@@ -68,10 +80,13 @@ TrackInfoObject::TrackInfoObject(const QDomNode &nodeHeader)
     m_iBitrate = XmlParse::selectNodeQString(nodeHeader, "Bitrate").toInt();
     m_iLength = XmlParse::selectNodeQString(nodeHeader, "Length").toInt();
     m_iTimesPlayed = XmlParse::selectNodeQString(nodeHeader, "TimesPlayed").toInt();
-    m_fBpm = XmlParse::selectNodeQString(nodeHeader, "Bpm").toFloat();
-    m_bBpmConfirm = XmlParse::selectNodeQString(nodeHeader, "BpmConfirm").toInt();
-    m_fBeatFirst = XmlParse::selectNodeQString(nodeHeader, "BeatFirst").toFloat();
+    m_fReplayGain = XmlParse::selectNodeQString(nodeHeader, "replaygain").toFloat();
     m_bHeaderParsed = false;
+    create_date = XmlParse::selectNodeQString(nodeHeader, "CreateDate");
+    if (create_date == "")
+        m_dCreateDate = fileInfo.created();
+    else
+        m_dCreateDate = QDateTime::fromString(create_date);
 
     // Mixxx <1.8 recorded track IDs in mixxxtrack.xml, but we are going to
     // ignore those. Tracks will get a new ID from the database.
@@ -79,9 +94,7 @@ TrackInfoObject::TrackInfoObject(const QDomNode &nodeHeader)
     m_iId = -1;
 
     m_fCuePoint = XmlParse::selectNodeQString(nodeHeader, "CuePoint").toFloat();
-
-    m_pVisualWave = 0;
-    m_dVisualResampleRate = 0;
+    m_bPlayed = false;
 
     //m_pWave = XmlParse::selectNodeHexCharArray(nodeHeader, QString("WaveSummaryHex"));
 
@@ -91,12 +104,11 @@ TrackInfoObject::TrackInfoObject(const QDomNode &nodeHeader)
     m_bLocationChanged = false;
 }
 
-void TrackInfoObject::populateLocation(QFileInfo& fileInfo) {
+void TrackInfoObject::populateLocation(const QFileInfo& fileInfo) {
     m_sFilename = fileInfo.fileName();
     m_sLocation = fileInfo.absoluteFilePath();
     m_sDirectory = fileInfo.absolutePath();
     m_iLength = fileInfo.size();
-    m_bExists = fileInfo.exists();
 }
 
 void TrackInfoObject::initialize(bool parseHeader) {
@@ -112,28 +124,34 @@ void TrackInfoObject::initialize(bool parseHeader) {
     m_iDuration = 0;
     m_iBitrate = 0;
     m_iTimesPlayed = 0;
-    m_fBpm = 0.;
-    m_bBpmConfirm = false;
+    m_bPlayed = false;
+    m_fReplayGain = 0.;
     m_bIsValid = false;
     m_bHeaderParsed = false;
-    m_fBeatFirst = -1.;
     m_iId = -1;
-    m_pVisualWave = 0;
     m_iSampleRate = 0;
     m_iChannels = 0;
     m_fCuePoint = 0.0f;
-    m_dVisualResampleRate = 0;
+    m_dCreateDate = m_dateAdded = QDateTime::currentDateTime();
+    m_Rating = 0;
+    m_key = "";
+    m_bBpmLock = false;
 
     // parse() parses the metadata from file. This is not a quick operation!
-    if (parseHeader)
+    if (parseHeader) {
         parse();
+    }
 }
 
 TrackInfoObject::~TrackInfoObject() {
+    //qDebug() << "~TrackInfoObject()" << m_iId << getInfo();
+    delete m_waveform;
+    delete m_waveformSummary;
 }
 
 void TrackInfoObject::doSave() {
-    emit(save());
+    //qDebug() << "TIO::doSave()" << getInfo();
+    emit(save(this));
 }
 
 bool TrackInfoObject::isValid() const {
@@ -148,6 +166,7 @@ void TrackInfoObject::writeToXML( QDomDocument &doc, QDomElement &header )
 {
     QMutexLocker lock(&m_qMutex);
 
+    QString create_date;
     XmlParse::addElement( doc, header, "Filename", m_sFilename );
     //XmlParse::addElement( doc, header, "Filepath", m_sFilepath );
     XmlParse::addElement( doc, header, "Title", m_sTitle );
@@ -160,19 +179,16 @@ void TrackInfoObject::writeToXML( QDomDocument &doc, QDomElement &header )
     XmlParse::addElement( doc, header, "Bitrate", QString("%1").arg(m_iBitrate));
     XmlParse::addElement( doc, header, "Length", QString("%1").arg(m_iLength) );
     XmlParse::addElement( doc, header, "TimesPlayed", QString("%1").arg(m_iTimesPlayed) );
-    XmlParse::addElement( doc, header, "Bpm", QString("%1").arg(m_fBpm) );
-    XmlParse::addElement( doc, header, "BpmConfirm", QString("%1").arg(m_bBpmConfirm) );
-    XmlParse::addElement( doc, header, "BeatFirst", QString("%1").arg(m_fBeatFirst) );
+    XmlParse::addElement( doc, header, "replaygain", QString("%1").arg(m_fReplayGain) );
     XmlParse::addElement( doc, header, "Id", QString("%1").arg(m_iId) );
     XmlParse::addElement( doc, header, "CuePoint", QString::number(m_fCuePoint) );
+    XmlParse::addElement( doc, header, "CreateDate", m_dCreateDate.toString() );
     //if (m_pWave) {
-        //XmlParse::addHexElement(doc, header, "WaveSummaryHex", m_pWave);
+    //XmlParse::addHexElement(doc, header, "WaveSummaryHex", m_pWave);
     //}
 
 }
 
-static void doNothing(TrackInfoObject* pTrack) {}
-
 int TrackInfoObject::parse()
 {
     // Add basic information derived from the filename:
@@ -219,7 +235,7 @@ QString TrackInfoObject::getDurationStr() const
     int iDuration = m_iDuration;
     lock.unlock();
 
-    return MixxxUtils::secondsToMinutes(iDuration);
+    return MixxxUtils::secondsToMinutes(iDuration, true);
 }
 
 void TrackInfoObject::setLocation(QString location)
@@ -255,29 +271,79 @@ QString TrackInfoObject::getFilename()  const
     return m_sFilename;
 }
 
+QDateTime TrackInfoObject::getCreateDate() const
+{
+    QMutexLocker lock(&m_qMutex);
+    QDateTime create_date = QDateTime(m_dCreateDate);
+    return create_date;
+}
+
 bool TrackInfoObject::exists()  const
 {
     QMutexLocker lock(&m_qMutex);
-    return m_bExists;
+    // return here a fresh calculated value to be sure
+    // the file is not deleted or gone with an USB-Stick
+    // because it will probably stop the Auto-DJ
+    return QFile::exists(m_sLocation);
 }
 
-
-float TrackInfoObject::getBpm() const
+float TrackInfoObject::getReplayGain() const
 {
     QMutexLocker lock(&m_qMutex);
-    return m_fBpm;
+    return m_fReplayGain;
 }
 
-void TrackInfoObject::setBpm(float f)
+void TrackInfoObject::setReplayGain(float f)
 {
     QMutexLocker lock(&m_qMutex);
-    bool dirty = m_fBpm != f;
-    m_fBpm = f;
+    bool dirty = m_fReplayGain != f;
+    m_fReplayGain = f;
+    //qDebug() << "Reported ReplayGain value: " << m_fReplayGain;
     if (dirty)
         setDirty(true);
     lock.unlock();
+    emit(ReplayGainUpdated(f));
+}
 
-    //Tell the GUI to update the bpm label...
+double TrackInfoObject::getBpm() const {
+    QMutexLocker lock(&m_qMutex);
+    if (!m_pBeats) {
+        return 0;
+    }
+    // getBpm() returns -1 when invalid.
+    double bpm = m_pBeats->getBpm();
+    if (bpm >= 0.0) {
+        return bpm;
+    }
+    return 0;
+}
+
+void TrackInfoObject::setBpm(double f) {
+    if (f < 0) {
+        return;
+    }
+
+    QMutexLocker lock(&m_qMutex);
+    // TODO(rryan): Assume always dirties.
+    bool dirty = false;
+    if (f == 0.0) {
+        // If the user sets the BPM to 0, we assume they want to clear the
+        // beatgrid.
+        setBeats(BeatsPointer());
+        dirty = true;
+    } else if (!m_pBeats) {
+        setBeats(BeatFactory::makeBeatGrid(this, f, 0));
+        dirty = true;
+    } else if (m_pBeats->getBpm() != f) {
+        m_pBeats->setBpm(f);
+        dirty = true;
+    }
+
+    if (dirty)
+        setDirty(true);
+
+    lock.unlock();
+    // Tell the GUI to update the bpm label...
     //qDebug() << "TrackInfoObject signaling BPM update to" << f;
     emit(bpmUpdated(f));
 }
@@ -287,16 +353,48 @@ QString TrackInfoObject::getBpmStr() const
     return QString("%1").arg(getBpm(), 3,'f',1);
 }
 
-bool TrackInfoObject::getBpmConfirm()  const
-{
+void TrackInfoObject::setBeats(BeatsPointer pBeats) {
     QMutexLocker lock(&m_qMutex);
-    return m_bBpmConfirm;
+
+    // This whole method is not so great. The fact that Beats is an ABC is
+    // limiting with respect to QObject and signals/slots.
+
+    QObject* pObject = NULL;
+    if (m_pBeats) {
+        pObject = dynamic_cast<QObject*>(m_pBeats.data());
+        if (pObject) {
+            disconnect(pObject, SIGNAL(updated()),
+                       this, SLOT(slotBeatsUpdated()));
+        }
+    }
+    m_pBeats = pBeats;
+    double bpm = 0.0;
+    if (m_pBeats) {
+        bpm = m_pBeats->getBpm();
+        pObject = dynamic_cast<QObject*>(m_pBeats.data());
+        if (pObject) {
+            connect(pObject, SIGNAL(updated()),
+                    this, SLOT(slotBeatsUpdated()));
+        }
+    }
+    setDirty(true);
+    lock.unlock();
+    emit(bpmUpdated(bpm));
+    emit(beatsUpdated());
 }
 
-void TrackInfoObject::setBpmConfirm(bool confirm)
-{
+BeatsPointer TrackInfoObject::getBeats() const {
     QMutexLocker lock(&m_qMutex);
-    m_bBpmConfirm = confirm;
+    return m_pBeats;
+}
+
+void TrackInfoObject::slotBeatsUpdated() {
+    QMutexLocker lock(&m_qMutex);
+    setDirty(true);
+    double bpm = m_pBeats->getBpm();
+    lock.unlock();
+    emit(bpmUpdated(bpm));
+    emit(beatsUpdated());
 }
 
 bool TrackInfoObject::getHeaderParsed()  const
@@ -322,6 +420,16 @@ QString TrackInfoObject::getInfo()  const
     return sInfo;
 }
 
+QDateTime TrackInfoObject::getDateAdded() const {
+    QMutexLocker lock(&m_qMutex);
+    return m_dateAdded;
+}
+
+void TrackInfoObject::setDateAdded(QDateTime dateAdded) {
+    QMutexLocker lock(&m_qMutex);
+    m_dateAdded = dateAdded;
+}
+
 int TrackInfoObject::getDuration()  const
 {
     QMutexLocker lock(&m_qMutex);
@@ -417,6 +525,22 @@ void TrackInfoObject::setGenre(QString s)
         setDirty(true);
 }
 
+QString TrackInfoObject::getComposer()  const
+{
+    QMutexLocker lock(&m_qMutex);
+    return m_sComposer;
+}
+
+void TrackInfoObject::setComposer(QString s)
+{
+    QMutexLocker lock(&m_qMutex);
+    s = s.trimmed();
+    bool dirty = m_sComposer != s;
+    m_sComposer = s;
+    if (dirty)
+        setDirty(true);
+}
+
 QString TrackInfoObject::getTrackNumber()  const
 {
     QMutexLocker lock(&m_qMutex);
@@ -439,11 +563,48 @@ int TrackInfoObject::getTimesPlayed()  const
     return m_iTimesPlayed;
 }
 
-void TrackInfoObject::incTimesPlayed()
+void TrackInfoObject::setTimesPlayed(int t)
 {
     QMutexLocker lock(&m_qMutex);
-    ++m_iTimesPlayed;
-    setDirty(true);
+    bool dirty = t != m_iTimesPlayed;
+    m_iTimesPlayed = t;
+    if (dirty)
+        setDirty(true);
+}
+
+void TrackInfoObject::incTimesPlayed()
+{
+    setPlayedAndUpdatePlaycount(true);
+}
+
+bool TrackInfoObject::getPlayed() const
+{
+    QMutexLocker lock(&m_qMutex);
+    bool bPlayed = m_bPlayed;
+    return bPlayed;
+}
+
+void TrackInfoObject::setPlayedAndUpdatePlaycount(bool bPlayed)
+{
+    QMutexLocker lock(&m_qMutex);
+    if (bPlayed) {
+        ++m_iTimesPlayed;
+        setDirty(true);
+    }
+    else if (m_bPlayed && !bPlayed) {
+        m_iTimesPlayed = math_max(0, m_iTimesPlayed - 1);
+        setDirty(true);
+    }
+    m_bPlayed = bPlayed;
+}
+
+void TrackInfoObject::setPlayed(bool bPlayed)
+{
+    QMutexLocker lock(&m_qMutex);
+    if (bPlayed != m_bPlayed) {
+        m_bPlayed = bPlayed;
+        setDirty(true);
+    }
 }
 
 QString TrackInfoObject::getComment() const
@@ -532,21 +693,6 @@ void TrackInfoObject::setBitrate(int i)
         setDirty(true);
 }
 
-void TrackInfoObject::setBeatFirst(float fBeatFirstPos)
-{
-    QMutexLocker lock(&m_qMutex);
-    bool dirty = m_fBeatFirst != fBeatFirstPos;
-    m_fBeatFirst = fBeatFirstPos;
-    if (dirty)
-        setDirty(true);
-}
-
-float TrackInfoObject::getBeatFirst() const
-{
-    QMutexLocker lock(&m_qMutex);
-    return m_fBeatFirst;
-}
-
 int TrackInfoObject::getId() const {
     QMutexLocker lock(&m_qMutex);
     return m_iId;
@@ -554,42 +700,20 @@ int TrackInfoObject::getId() const {
 
 void TrackInfoObject::setId(int iId) {
     QMutexLocker lock(&m_qMutex);
-    bool dirty = m_iId != iId;
+    // changing the Id does not make the track drity because the Id is always
+    // generated by the Database itself
     m_iId = iId;
-    if (dirty)
-        setDirty(true);
 }
 
-QVector<float> * TrackInfoObject::getVisualWaveform() {
-    QMutexLocker lock(&m_qMutex);
-    return m_pVisualWave;
-}
-
-void TrackInfoObject::setVisualResampleRate(double dVisualResampleRate) {
-    // Temporary, shared value that should not be saved. The only reason it
-    // exists on the TIO is a temporary hack, so it does not dirty the TIO.
-    QMutexLocker lock(&m_qMutex);
-    m_dVisualResampleRate = dVisualResampleRate;
-}
-
-double TrackInfoObject::getVisualResampleRate() {
-    QMutexLocker lock(&m_qMutex);
-    return m_dVisualResampleRate;
-}
 
+//TODO (vrince) remove clen-up when new summary is ready
+/*
 const QByteArray *TrackInfoObject::getWaveSummary()
 {
     QMutexLocker lock(&m_qMutex);
     return &m_waveSummary;
 }
 
-void TrackInfoObject::setVisualWaveform(QVector<float> *pWave) {
-    // The visual waveform is not serialized currently so it does not dirty a
-    // TIO.
-    QMutexLocker lock(&m_qMutex);
-    m_pVisualWave = pWave;
-}
-
 void TrackInfoObject::setWaveSummary(const QByteArray* pWave, bool updateUI)
 {
     QMutexLocker lock(&m_qMutex);
@@ -597,7 +721,7 @@ void TrackInfoObject::setWaveSummary(const QByteArray* pWave, bool updateUI)
     setDirty(true);
     lock.unlock();
     emit(wavesummaryUpdated(this));
-}
+}*/
 
 void TrackInfoObject::setURL(QString url)
 {
@@ -614,6 +738,32 @@ QString TrackInfoObject::getURL()
     return m_sURL;
 }
 
+Waveform* TrackInfoObject::getWaveform() {
+    return m_waveform;
+}
+
+Waveform* TrackInfoObject::getWaveformSummary() {
+    return m_waveformSummary;
+}
+
+// called from the AnalyserQueue Thread
+void TrackInfoObject::waveformSummaryNew() {
+    emit(waveformSummaryUpdated());
+}
+
+void TrackInfoObject::setAnalyserProgress(int progress) {
+    // progress in 0 .. 1000. QAtomicInt so no need for lock.
+    if (progress != m_analyserProgress) {
+        m_analyserProgress = progress;
+        emit(analyserProgress(progress));
+    }
+}
+
+int TrackInfoObject::getAnalyserProgress() const {
+    // QAtomicInt so no need for lock.
+    return m_analyserProgress;
+}
+
 void TrackInfoObject::setCuePoint(float cue)
 {
     QMutexLocker lock(&m_qMutex);
@@ -650,7 +800,7 @@ Cue* TrackInfoObject::addCue() {
 void TrackInfoObject::removeCue(Cue* cue) {
     QMutexLocker lock(&m_qMutex);
     disconnect(cue, 0, this, 0);
-    m_cuePoints.remove(cue);
+    m_cuePoints.removeOne(cue);
     setDirty(true);
     lock.unlock();
     emit(cuesUpdated());
@@ -674,7 +824,7 @@ void TrackInfoObject::setCuePoints(QList<Cue*> cuePoints) {
     while (it.hasNext()) {
         Cue* cue = it.next();
         connect(cue, SIGNAL(updated()),
-            this, SLOT(slotCueUpdated()));
+                this, SLOT(slotCueUpdated()));
     }
     setDirty(true);
     lock.unlock();
@@ -693,23 +843,25 @@ void TrackInfoObject::setChordData(Segmentation<QString> cd) {
 }
 
 void TrackInfoObject::setDirty(bool bDirty) {
+
     QMutexLocker lock(&m_qMutex);
     bool change = m_bDirty != bDirty;
     m_bDirty = bDirty;
     lock.unlock();
+    // qDebug() << "Track" << m_iId << getInfo() << (change? "changed" : "unchanged")
+    //          << "set" << (bDirty ? "dirty" : "clean");
     if (change) {
-        //qDebug() << "Track" << m_iId << "set" << (bDirty ? "dirty" : "clean");
         if (m_bDirty)
-            emit(dirty());
+            emit(dirty(this));
         else
-            emit(clean());
+            emit(clean(this));
     }
     // Emit a changed signal regardless if this attempted to set us dirty.
-    if (bDirty)
-        emit(changed());
+    if (bDirty) {
+        emit(changed(this));
+    }
 
-
-    //qDebug() << QString("TrackInfoObject %1 %2 set to %3").arg(m_iId).arg(m_sLocation).arg(m_bDirty ? "dirty" : "clean");
+    //qDebug() << QString("TrackInfoObject %1 %2 set to %3").arg(QString::number(m_iId), m_sLocation, m_bDirty ? "dirty" : "clean");
 }
 
 bool TrackInfoObject::isDirty() {
@@ -721,3 +873,42 @@ bool TrackInfoObject::locationChanged() {
     QMutexLocker lock(&m_qMutex);
     return m_bLocationChanged;
 }
+
+int TrackInfoObject::getRating() const{
+    QMutexLocker lock(&m_qMutex);
+    return m_Rating;
+}
+
+void TrackInfoObject::setRating (int rating){
+    QMutexLocker lock(&m_qMutex);
+    bool dirty = rating != m_Rating;
+    m_Rating = rating;
+    if (dirty)
+        setDirty(true);
+}
+
+QString TrackInfoObject::getKey() const{
+    QMutexLocker lock(&m_qMutex);
+    return m_key;
+}
+
+void TrackInfoObject::setKey(QString key){
+    QMutexLocker lock(&m_qMutex);
+    bool dirty = key != m_key;
+    m_key = key;
+    if (dirty)
+        setDirty(true);
+}
+
+void TrackInfoObject::setBpmLock(bool bpmLock) {
+    QMutexLocker lock(&m_qMutex);
+    bool dirty = bpmLock != m_bBpmLock;
+    m_bBpmLock = bpmLock;
+    if (dirty)
+        setDirty(true);
+}
+
+bool TrackInfoObject::hasBpmLock() const {
+    QMutexLocker lock(&m_qMutex);
+    return m_bBpmLock;
+}
