@@ -18,18 +18,18 @@
 #ifndef TRACKINFOOBJECT_H
 #define TRACKINFOOBJECT_H
 
+#include <QAtomicInt>
 #include <QList>
+#include <QDateTime>
 #include <QObject>
 #include <QFileInfo>
-#include <q3memarray.h>
-#include <q3valuelist.h>
 #include <QMutex>
 #include <QVector>
 #include <QSharedPointer>
 #include <QWeakPointer>
 
 #include "defs.h"
-
+#include "track/beats.h"
 #include "library/dao/cue.h"
 
 class QString;
@@ -39,6 +39,7 @@ class QDomNode;
 class ControlObject;
 class TrackPlaylist;
 class Cue;
+class Waveform;
 
 class TrackInfoObject;
 
@@ -54,8 +55,8 @@ public:
     /** Initialize a new track with the filename. */
     TrackInfoObject(const QString sLocation="", bool parseHeader=true);
     // Initialize track with a QFileInfo class
-    TrackInfoObject(QFileInfo& fileInfo, bool parseHeader=true);
-    /** Creates a new track given information from the xml file. */
+    TrackInfoObject(const QFileInfo& fileInfo, bool parseHeader=true);
+    // Creates a new track given information from the xml file.
     TrackInfoObject(const QDomNode &);
     virtual ~TrackInfoObject();
 
@@ -66,34 +67,60 @@ public:
 
     /** Returns the duration in seconds */
     int getDuration() const;
+    /** Set duration in seconds */
+    void setDuration(int);
     /** Returns the duration as a string: H:MM:SS */
     QString getDurationStr() const;
 
     // Accessors for various stats of the file on disk. These are auto-populated
     // when the TIO is constructed, or when setLocation() is called.
 
+    Q_PROPERTY(QString artist READ getArtist WRITE setArtist)
+    Q_PROPERTY(QString title READ getTitle WRITE setTitle)
+    Q_PROPERTY(QString album READ getAlbum WRITE setAlbum)
+    Q_PROPERTY(QString genre READ getGenre WRITE setGenre)
+    Q_PROPERTY(QString composer READ getComposer WRITE setComposer)
+    Q_PROPERTY(QString year READ getYear WRITE setYear)
+    Q_PROPERTY(QString track_number READ getTrackNumber WRITE setTrackNumber)
+    Q_PROPERTY(int times_played READ getTimesPlayed)
+    Q_PROPERTY(QString comment READ getComment WRITE setComment)
+    Q_PROPERTY(double bpm READ getBpm WRITE setBpm)
+    Q_PROPERTY(QString bpmFormatted READ getBpmStr STORED false)
+    Q_PROPERTY(QString key READ getKey WRITE setKey)
+    Q_PROPERTY(int duration READ getDuration WRITE setDuration)
+    Q_PROPERTY(QString durationFormatted READ getDurationStr STORED false)
+
+
     // Returns absolute path to the file, including the filename.
     QString getLocation() const;
     // Returns the absolute path to the directory containing the file
     QString getDirectory() const;
     // Returns the filename of the file.
     QString getFilename() const;
+    // Returns file creation date
+    QDateTime getCreateDate() const;
     // Returns the length of the file in bytes
     int getLength() const;
     // Returns whether the file exists on disk or not. Updated as of the time
     // the TrackInfoObject is created, or when setLocation() is called.
     bool exists() const;
 
+
+
+    /** Returns ReplayGain*/
+    float getReplayGain() const;
+    /** Set ReplayGain*/
+    void setReplayGain(float);
     /** Returns BPM */
-    float getBpm() const;
+    double getBpm() const;
     /** Set BPM */
-    void setBpm(float);
+    void setBpm(double);
     /** Returns BPM as a string */
     QString getBpmStr() const;
-    /** Retruns if BPM was confirmed (edited or verified manually) */
-    bool getBpmConfirm() const;
-    /** Set BPM confidence */
-    void setBpmConfirm(bool confirm=true);
+    // A track with a locked BPM will not be re-analyzed by the beats or bpm
+    // analyzer.
+    void setBpmLock(bool hasLock);
+    bool hasBpmLock() const;
     bool getHeaderParsed() const;
     void setHeaderParsed(bool parsed = true);
     /** Returns the user comment */
@@ -110,10 +137,6 @@ public:
     QString getBitrateStr() const;
     /** Sets the bitrate */
     void setBitrate(int);
-    /** Sets first beat pos */
-    void setBeatFirst(float);
-    /** Get first beat pos */
-    float getBeatFirst() const;
     /** Set sample rate */
     void setSampleRate(int iSampleRate);
     /** Get sample rate */
@@ -124,8 +147,9 @@ public:
     int getChannels() const;
     /** Output a formatted string with all the info */
     QString getInfo() const;
-    /** Set duration in seconds */
-    void setDuration(int);
+
+    QDateTime getDateAdded() const;
+    void setDateAdded(QDateTime dateAdded);
 
     /** Getter/Setter methods for metadata */
     /** Return title */
@@ -148,39 +172,53 @@ public:
     QString getGenre() const;
     /** Set genre */
     void setGenre(QString);
+    /** Return composer */
+    QString getComposer() const;
+    /** Set composer */
+    void setComposer(QString);
     /** Return Track Number */
     QString getTrackNumber() const;
     /** Set Track Number */
     void setTrackNumber(QString);
     /** Return number of times the track has been played */
     int getTimesPlayed() const;
+    /** Set number of times the track has been played */
+    void setTimesPlayed(int t);
     /** Increment times played with one */
     void incTimesPlayed();
+    /** Returns true if track has been played this instance*/
+    bool getPlayed() const;
+    /** Set played status and increment or decrement playcount. */
+    void setPlayedAndUpdatePlaycount(bool);
+    /** Set played status without affecting the playcount */
+    void setPlayed(bool bPlayed);
 
     int getId() const;
 
+    /** Returns rating */
+    int getRating() const;
+    /** Sets rating */
+    void setRating(int);
+
+    /** Returns KEY_CODE */
+    QString getKey() const;
+    /** Set KEY_CODE */
+    void setKey(QString);
 
     /** Get URL for track */
     QString getURL();
     /** Set URL for track */
     void setURL(QString url);
-    /** Set pointer to visual waveform data */
-    void setVisualWaveform(QVector<float> *pWave);
-    /** Get pointer to visual waveform data */
-    QVector<float> * getVisualWaveform();
 
-    /** Set and get this track's desired visual resample rate */
-    void setVisualResampleRate(double dVisualResampleRate);
-    double getVisualResampleRate();
+    Waveform* getWaveform();
+    void waveformNew();
 
-    /** Set pointer to waveform summary -- updates UI by default */
-    void setWaveSummary(const QByteArray* pWave, bool updateUI = true);
+    Waveform* getWaveformSummary();
+    const Waveform* getWaveformSummary() const;
+    void waveformSummaryNew();
 
-    /** Returns a pointer to waveform summary */
-    const QByteArray* getWaveSummary();
-
-    /** Set pointer to ControlObject holding BPM value in engine */
-    void setBpmControlObject(ControlObject *p);
+    void setAnalyserProgress(int progress);
+    int getAnalyserProgress() const;
 
     /** Save the cue point (in samples... I think) */
     void setCuePoint(float cue);
@@ -205,28 +243,41 @@ public:
     /** Set the track's full file path */
     void setLocation(QString location);
 
+    // Get the track's Beats list
+    BeatsPointer getBeats() const;
+
+    // Set the track's Beats
+    void setBeats(BeatsPointer beats);
+
     const Segmentation<QString>* getChordData();
     void setChordData(Segmentation<QString> cd);
 
   public slots:
     void slotCueUpdated();
 
-  signals:
-    void wavesummaryUpdated(TrackInfoObject*);
+signals:
+    void waveformUpdated();
+    void waveformSummaryUpdated();
+    void analyserProgress(int progress);
     void bpmUpdated(double bpm);
+    void beatsUpdated();
+    void ReplayGainUpdated(double replaygain);
     void cuesUpdated();
-    void changed();
-    void dirty();
-    void clean();
-    void save();
+    void changed(TrackInfoObject* pTrack);
+    void dirty(TrackInfoObject* pTrack);
+    void clean(TrackInfoObject* pTrack);
+    void save(TrackInfoObject* pTrack);
 
-  private:
+private slots:
+    void slotBeatsUpdated();
+
+private:
 
     // Common initialization function between all TIO constructors.
     void initialize(bool parseHeader);
 
     // Initialize all the location variables.
-    void populateLocation(QFileInfo& fileInfo);
+    void populateLocation(const QFileInfo& fileInfo);
 
     // Method for parsing information from knowing only the file name.  It
     // assumes that the filename is written like: "artist - trackname.xxx"
@@ -255,8 +306,6 @@ public:
     QString m_sDirectory;
     // Length of track in bytes
     int m_iLength;
-    // Flag which indicates whether the file exists or not.
-    bool m_bExists;
 
     /** Metadata */
     /** Album */
@@ -267,6 +316,8 @@ public:
     QString m_sTitle;
     /** Genre */
     QString m_sGenre;
+    /** Composer */
+    QString m_sComposer;
     /** Year */
     QString m_sYear;
     /** Track Number */
@@ -284,34 +335,35 @@ public:
     int m_iSampleRate;
     /** Number of channels */
     int m_iChannels;
+    /**Track rating */
+    int m_Rating;
     /** Bitrate, number of kilobits per second of audio in the track*/
     int m_iBitrate;
     /** Number of times the track has been played */
     int m_iTimesPlayed;
-    /** Beat per minutes (BPM) */
-    float m_fBpm;
-    /** Minimum BPM range. If this is 0.0, then the config min BPM will be used */
-    float m_fMinBpm;
-    /** Maximum BPM range. If this is 0.0, then the config max BPM will be used */
-    float m_fMaxBpm;
-    /** True if BPM is confirmed */
-    bool m_bBpmConfirm;
+    /** Replay Gain volume */
+    float m_fReplayGain;
+    /** Has this track been played this sessions? */
+    bool m_bPlayed;
     /** True if header was parsed */
     bool m_bHeaderParsed;
-    /** Position of first beat in song */
-    float m_fBeatFirst;
     /** Id. Unique ID of track */
     int m_iId;
     /** Cue point in samples or something */
     float m_fCuePoint;
+    /** Date. creation date of file */
+    QDateTime m_dCreateDate;
+    // Date the track was added to the library
+    QDateTime m_dateAdded;
+
+    QString m_key;
+
+    /** BPM lock **/
+    bool m_bBpmLock;
 
     // The list of cue points for the track
     QList<Cue*> m_cuePoints;
 
-    /** Pointer to visual waveform info */
-    QVector<float> *m_pVisualWave;
-    /** Wave summary info */
-    QByteArray m_waveSummary;
 
     /** Mutex protecting access to object */
     mutable QMutex m_qMutex;
@@ -319,9 +371,17 @@ public:
     /** True if object contains valid information */
     bool m_bIsValid;
 
-    double m_dVisualResampleRate;
     Segmentation<QString> m_chordData;
 
+    // Storage for the track's beats
+    BeatsPointer m_pBeats;
+
+    //Visual waveform data
+    Waveform* const m_waveform;
+    Waveform* const m_waveformSummary;
+
+    QAtomicInt m_analyserProgress; // in 0.1%
+
     friend class TrackDAO;
 };
 
