@@ -18,6 +18,18 @@
 #ifndef UPGRADE_H
 #define UPGRADE_H
 
-ConfigObject<ConfigValue>* versionUpgrade();
+class Upgrade
+{
+    public:
+        Upgrade();
+        ~Upgrade();
+        ConfigObject<ConfigValue>* versionUpgrade(const QString& settingsPath);
+        bool isFirstRun() { return m_bFirstRun; };
+        bool isUpgraded() { return m_bUpgraded; };
+    private:
+        bool askReanalyzeBeats();
+        bool m_bFirstRun;
+        bool m_bUpgraded;
+};
 
 #endif
