@@ -15,42 +15,43 @@
 *                                                                         *
 ***************************************************************************/
 
-#include "wknob.h"
-#include "wpixmapstore.h"
-
 #include <QPixmap>
 #include <QtDebug>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QPaintEvent>
 
-WKnob::WKnob(QWidget * parent, float defaultValue)
-    : WAbstractControl(parent, defaultValue)
-{
-    m_pPixmaps = 0;
-    m_pPixmapBack = 0;
-    m_bDisabledLoaded = false;
-    setPositions(0);
+#include "widget/wknob.h"
+
+#include "defs.h"
+#include "widget/wpixmapstore.h"
+
+WKnob::WKnob(QWidget * parent)
+        : WAbstractControl(parent),
+          m_iPos(0),
+          m_iNoPos(0),
+          m_pPixmaps(NULL),
+          m_pPixmapBack(NULL),
+          m_bDisabledLoaded(false) {
 }
 
 WKnob::~WKnob()
 {
     resetPositions();
+    if (m_pPixmapBack) {
+       WPixmapStore::deletePixmap(m_pPixmapBack);
+    }
 }
 
 void WKnob::setup(QDomNode node)
 {
-    WWidget::setup(node);
-
     // Set background pixmap if available
     if (!selectNode(node, "BackPath").isNull())
         setPixmapBackground(getPath(selectNodeQString(node, "BackPath")));
 
     // Number of states. Depends if disabled pics are defined as well
-    if (!selectNode(node, "DisabledPath").isNull())
-        setPositions(selectNodeInt(node, "NumberStates"),true);
-    else
-        setPositions(selectNodeInt(node, "NumberStates"),false);
+    setPositions(selectNodeInt(node, "NumberStates"),
+                 !selectNode(node, "DisabledPath").isNull());
 
     // Load knob pixmaps
     QString path = selectNodeQString(node, "Path");
@@ -69,11 +70,11 @@ void WKnob::setup(QDomNode node)
 
 void WKnob::setPositions(int iNoPos, bool bIncludingDisabled)
 {
+    resetPositions();
+
     m_iNoPos = iNoPos;
     m_iPos = 0;
 
-    resetPositions();
-
     if (m_iNoPos>0)
     {
         int pics = m_iNoPos;
@@ -90,12 +91,17 @@ void WKnob::resetPositions()
 {
     if (m_pPixmaps)
     {
-        for (int i=0; i<m_iNoPos; i++)
-            if (m_pPixmaps[i])
+        int pics = m_iNoPos;
+        if( m_bDisabledLoaded ){
+            pics *= 2;
+        }
+        for (int i=0; i<pics; i++) {
+            if (m_pPixmaps[i]) {
                 WPixmapStore::deletePixmap(m_pPixmaps[i]);
-
-        //WPixmapStore::deletePixmap(m_pPixmaps);
-        m_pPixmaps = 0;
+            }
+        }
+        delete [] m_pPixmaps;
+        m_pPixmaps = NULL;
     }
 }
 
@@ -110,6 +116,9 @@ void WKnob::setPixmap(int iPos, const QString &filename)
 void WKnob::setPixmapBackground(const QString &filename)
 {
     // Load background pixmap
+   if (m_pPixmapBack) {
+       WPixmapStore::deletePixmap(m_pPixmapBack);
+   }
     m_pPixmapBack = WPixmapStore::getPixmap(filename);
     if (!m_pPixmapBack)
         qDebug() << "WKnob: Error loading background pixmap:" << filename;
@@ -118,8 +127,21 @@ void WKnob::setPixmapBackground(const QString &filename)
 void WKnob::mouseMoveEvent(QMouseEvent * e)
 {
     if (!m_bRightButtonPressed) {
-        m_fValue += (m_dStartValue-e->y());
-        m_dStartValue = e->y();
+        QPoint cur(e->globalPos());
+        QPoint diff(cur - m_startPos);
+        double dist = sqrt(static_cast<double>(diff.x() * diff.x() + diff.y() * diff.y()));
+        bool y_dominant = abs(diff.y()) > abs(diff.x());
+
+        // if y is dominant, then thread an increase in dy as negative (y is
+        // pointed downward). Otherwise, if y is not dominant and x has
+        // decreased, then thread it as negative.
+        if ((y_dominant && diff.y() > 0) || (!y_dominant && diff.x() < 0)) {
+            dist = -dist;
+        }
+
+        m_fValue += dist;
+        QCursor::setPos(m_startPos);
+
         if (m_fValue>127.)
             m_fValue = 127.;
         else if (m_fValue<0.)
@@ -131,31 +153,50 @@ void WKnob::mouseMoveEvent(QMouseEvent * e)
 
 void WKnob::mousePressEvent(QMouseEvent * e)
 {
-    m_dStartValue = e->y();
-
-    if (e->button() == Qt::RightButton)
-    {
-        reset();
+    switch (e->button()) {
+    case Qt::RightButton:
+        emit(valueReset());
         m_bRightButtonPressed = true;
+        break;
+    case Qt::LeftButton:
+    case Qt::MidButton:
+        m_startPos = e->globalPos();
+        QApplication::setOverrideCursor(Qt::BlankCursor);
+        break;
+    default:
+        break;
     }
 }
 
 void WKnob::mouseReleaseEvent(QMouseEvent * e)
 {
-    if (e->button()==Qt::LeftButton)
+    switch (e->button()) {
+    case Qt::LeftButton:
+    case Qt::MidButton:
+        QCursor::setPos(m_startPos);
+        QApplication::restoreOverrideCursor();
         emit(valueChangedLeftUp(m_fValue));
-    else if (e->button()==Qt::RightButton)
+        break;
+    case Qt::RightButton:
         m_bRightButtonPressed = false;
         //emit(valueChangedRightUp(m_fValue));
+        break;
+    default:
+        break;
+    }
 
     update();
 }
 
 void WKnob::wheelEvent(QWheelEvent *e)
-{   
+{
     double wheelDirection = ((QWheelEvent *)e)->delta() / 120.;
-    double newValue = getValue() + (wheelDirection);
-    this->updateValue(newValue);
+    double newValue = getValue() + wheelDirection;
+
+    // Clamp to [0.0, 127.0]
+    newValue = math_max(0.0, math_min(127.0, newValue));
+
+    updateValue(newValue);
 
     e->accept();
 
@@ -164,7 +205,7 @@ void WKnob::wheelEvent(QWheelEvent *e)
 
 void WKnob::paintEvent(QPaintEvent *)
 {
-    if (m_pPixmaps>0)
+    if (m_pPixmaps)
     {
         int idx = (int)(((m_fValue-64.)*(((float)m_iNoPos-1.)/127.))+((float)m_iNoPos/2.));
         // Range check
