@@ -20,11 +20,11 @@ class WLibraryTableView : public QTableView, public virtual LibraryView {
                       ConfigObject<ConfigValue>* pConfig,
                       ConfigKey vScrollBarPosKey);
     virtual ~WLibraryTableView();
-    virtual void setup(QDomNode node);
+    virtual void moveSelection(int delta);
 
   signals:
     void loadTrack(TrackPointer pTrack);
-    void loadTrackToPlayer(TrackPointer pTrack, int player);
+    void loadTrackToPlayer(TrackPointer pTrack, QString group, bool play = false);
 
   public slots:
     void saveVScrollBarPos();
